/**
 * 
 */
package uoc.tfc.gien;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.location.Location;
import android.os.Bundle;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.Iterator;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.common.api.Status;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;

import uoc.tfc.gien.entities.Experience;
import uoc.tfc.gien.entities.geo.GPSPosition;
import uoc.tfc.gien.entities.geo.GeoObject;
import uoc.tfc.gien.entities.geo.Layer;
import uoc.tfc.gien.entities.geo.SinglePointObject;
import uoc.tfc.gien.geofencing.Geofencing;


/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class CoreService extends Service implements GoogleApiClient.ConnectionCallbacks, GoogleApiClient.OnConnectionFailedListener, LocationListener, ResultCallback<Status> {
	
	final String TAG = "CoreService";
	
	private GoogleApiClient mGoogleApiClient;
    private LocationRequest mLocationRequest;
    private int mGPSRefreshInterval;
    private int connectionInterval;
    private Geofencing geoFencing;
    
    private BroadcastReceiver receiverMapChanging;
    
    private GlobalApplicationData app;
	
	public void showToast(String msg) {
		Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
	}
	
	/**
	 * @return the mGPSRefreshInterval
	 */
	public int getGPSRefreshInterval() {
		return mGPSRefreshInterval;
	}

	/**
	 * @param mGPSRefreshInterval the mGPSRefreshInterval to set
	 */
	public void setGPSRefreshInterval(int mGPSRefreshInterval) {
		this.mGPSRefreshInterval = mGPSRefreshInterval;
	}

	@Override
	public void onCreate() {
		app = (GlobalApplicationData) this.getApplicationContext();
		
		Intent i = new Intent(this, uoc.tfc.gien.StartPointActivity.class);
		PendingIntent resultPendingIntent = PendingIntent.getActivity(getApplication(), 0, i, PendingIntent.FLAG_UPDATE_CURRENT);
		
		Notification notification = new Notification.Builder(this)
			.setContentTitle(app.getGieInfo().getName())
			.setContentText(app.getGieInfo().getDescription())
			.setContentIntent(resultPendingIntent)
			.setSmallIcon(R.drawable.ic_notification)
			.build(); //min API 16
			//.getNotification(); //API 11 a 16?
		
		startForeground(-1, notification);	// porta el servei a foreground i el manté obert encara que es tanquin totes les activities
		
		mGoogleApiClient = new GoogleApiClient.Builder(this)
        	.addApi(LocationServices.API)
        	.addConnectionCallbacks(this)
        	.addOnConnectionFailedListener(this)
        	.build();
		
		SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(this);
		
		app.setSoundNotifications(sharedPref.getBoolean("allow_sound_notifications", true));
		
		if (sharedPref.getBoolean("low_gps_refresh", false)) {
			this.setGPSRefreshInterval(Constants.LOW_GPS_REFRESH_INTERVAL_MS);
		} else {
			this.setGPSRefreshInterval(Constants.HIGH_GPS_REFRESH_INTERVAL_MS);
		}
		
		// adjusts network connection interval
		Experience exp = app.getExperience();
		if (app.isOnlineGie() && exp.isConnectRegularly() && sharedPref.getBoolean("low_network_use", true)) {
			int expConInterval = exp.getConnectionInterval(); 
			if (expConInterval < Constants.LOW_NETWORK_USE_INTERVAL_MS) {
				connectionInterval = Constants.LOW_NETWORK_USE_INTERVAL_MS;
			} else {
				connectionInterval = expConInterval;
			}
		}
		
		////////////////////////////////////////////////////////////////////////
		// TO DO: set alarm and call locationsenderintentservice or use a thread
		////////////////////////////////////////////////////////////////////////
		
		IntentFilter intentFilterMapChanging = new IntentFilter();
		intentFilterMapChanging.addAction("uoc.tfc.gien.MAP_CHANGING");
		receiverMapChanging = new BroadcastReceiver() {
			@Override
			public void onReceive(Context context, Intent intent) {
				if (intent.getStringExtra("status").compareTo("start")==0) {
					geoFencing.removeGeofences();
				} else {
					if (intent.getStringExtra("status").compareTo("end")==0) {
						refreshGeofences();
					}
				}
			}
		};
		registerReceiver(receiverMapChanging, intentFilterMapChanging);
		
	}
	
	@Override
	public int onStartCommand(Intent intencion, int flags, int idArranque) {
		mGoogleApiClient.connect();
		return START_STICKY; // flag indicating the service will restart in case it stops unexpectedly
	}
	
	@Override
	public void onDestroy() {
		
		((NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE)).cancelAll();
		
		if (geoFencing!=null) {
			geoFencing.removeGeofences();
		}
		
		if (receiverMapChanging!=null) {
			unregisterReceiver(receiverMapChanging);
		}
		
		try {
			if (mGoogleApiClient!=null) {
				if(mGoogleApiClient.isConnected()==true) {
					mGoogleApiClient.disconnect();
				}
			}
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}
	}

	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}


	/**
	 * Broadcasts location. MapActivity will use this location.
	 * @param location
	 */
	private void broadcastLocation(Location location) {
		Intent i = new Intent("uoc.tfc.gien.LOCATION_CHANGE");
		i.putExtra("location", new double[]{location.getLatitude(),location.getLongitude()});
		sendBroadcast(i);
	}
	
	@Override
	public void onLocationChanged(Location location) {
		broadcastLocation(location);
		
		GPSPosition pos = app.getLocation();
		pos.setLatitude(location.getLatitude());
		pos.setLongitude(location.getLongitude());
	}

	@Override
	public void onConnectionFailed(ConnectionResult cResult) {
		showToast(TAG+"GoogleApiClient connection has failed: " + cResult.toString());
	}

	@Override
	public void onConnected(Bundle bundle) {
		mLocationRequest = LocationRequest.create();
        mLocationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
        mLocationRequest.setInterval(getGPSRefreshInterval());

        LocationServices.FusedLocationApi.requestLocationUpdates(mGoogleApiClient, mLocationRequest, this);
        
        geoFencing = Geofencing.getInstance(this, mGoogleApiClient);
        app.setGeofencing(geoFencing);
        
		refreshGeofences();
        
        LocationServices.GeofencingApi.addGeofences(
				mGoogleApiClient,
				geoFencing.getGeofencingRequest(),
				geoFencing.getGeofencePendingIntent()
		).setResultCallback(this);
        
	}
	
	private void refreshGeofences() {
		geoFencing.removeGeofences();
		ArrayList<Layer> layers = app.getExperience().getMap().getLayers();
		Iterator<Layer> it = layers.iterator();
		Layer l = null;
		while (it.hasNext()) {
			l = it.next();
			Iterator<GeoObject> itg = l.getObjects().iterator();
			GeoObject gObj = null;
			while (itg.hasNext()) {
				gObj = itg.next();
				if (gObj.getType()==GeoObject.Type.SINGLE_POINT) {
					geoFencing.addGeofence((SinglePointObject)gObj);
				}
			}
		}
	}
	
	public void changeLocationRequestInterval(long ms) {
		mLocationRequest.setInterval(ms);
		LocationServices.FusedLocationApi.requestLocationUpdates(mGoogleApiClient, mLocationRequest, this);
	}

	@Override
	public void onConnectionSuspended(int code) {
		showToast(TAG+" - GoogleApiClient connection has been suspend: " + Integer.toString(code));
	}


	// result from geofencing ops
	@Override
	public void onResult(Status s) {
		//showToast(TAG+": "+s.toString());
	}
	

}
