/**
 * 
 */
package uoc.tfc.gien;


import java.io.File;

import java.util.TreeMap;

import org.json.JSONObject;

import uoc.tfc.gien.database.FavouritesSQLite;
import uoc.tfc.gien.database.GieSQLite;
import uoc.tfc.gien.entities.Experience;
import uoc.tfc.gien.entities.GieInfo;
import uoc.tfc.gien.entities.geo.GPSPosition;

import uoc.tfc.gien.entities.geo.SinglePointObject;
import uoc.tfc.gien.geofencing.Geofencing;
import uoc.tfc.gien.storage.Storage;
import uoc.tfc.gien.ui.MapActivity;
import uoc.tfc.gien.entities.interactive.Select;

import android.app.Application;
import android.content.Context;
import android.content.Intent;


/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class GlobalApplicationData extends Application {
	
	private Experience gie;
	private Geofencing geofencing;
	private GieInfo gieInfo;
	
	private String serverUrl;		//if this has a value, this url is verified and contains a '/' at the end
	private boolean onlineGie;		//indicates if the gie is in a server
	private boolean fromFavorites;	//indicates if the gie was loaded from favorites list, used for disable "add to favorite" option in menu
	
	private boolean storageAccess;	// indicates if the storage subsystem is working ok
	private File gieFolder;
	private GieSQLite database;		// gie database
	private FavouritesSQLite FavoritesDatabase;	//Favorites database
	
	private GPSPosition location;	// indicates user's location
	
	private TreeMap<Integer, Select> queuedSelects;
	private TreeMap<String, SinglePointObject> geofenceSPOMap;	// a way to link a geofence with a SinglePointObject.
																// Helps to find the SinglePointObject data when a geofence event occurs
	
	private boolean soundNotifications;		// indicates if is allowed to play sound when notifications arrives
	
	
	@Override
	public void onCreate() {
		super.onCreate();
		this.setExperience(new Experience());
		this.setGieInfo(new GieInfo());
		setStorageAccess(Storage.init());
		this.setLocation(new GPSPosition());
		this.setDatabase(GieSQLite.getInstance(this));
		this.setFavoritesDatabase(FavouritesSQLite.getInstance(this));
		this.setQueuedSelects(new TreeMap<Integer,Select>());
		this.geofenceSPOMap = new TreeMap<String, SinglePointObject>();
	}
	
	/**
	 * @return the experience
	 */
	public Experience getExperience() {
		return gie;
	}
	
	/**
	 * @param experience the experience to set
	 */
	public void setExperience(Experience experience) {
		this.gie = experience;
	}

	/**
	 * @return the gieInfo
	 */
	public GieInfo getGieInfo() {
		return gieInfo;
	}

	/**
	 * @param gieInfo the gieInfo to set
	 */
	public void setGieInfo(GieInfo gieInfo) {
		this.gieInfo = gieInfo;
	}
	
	/**
	 * @return the serverUrl
	 */
	public String getServerUrl() {
		return serverUrl;
	}

	/**
	 * @param serverUrl the serverUrl to set
	 */
	public void setServerUrl(String serverUrl) {
		this.serverUrl = serverUrl;
	}
	
	/**
	 * @return the onlineGie
	 */
	public boolean isOnlineGie() {
		return onlineGie;
	}

	/**
	 * @param onlineGie the onlineGie to set
	 */
	public void setOnlineGie(boolean onlineGie) {
		this.onlineGie = onlineGie;
	}
	
	

	/**
	 * @return the geofencing
	 */
	public Geofencing getGeofencing() {
		return geofencing;
	}

	/**
	 * @param geofencing the geofencing to set
	 */
	public void setGeofencing(Geofencing geofencing) {
		this.geofencing = geofencing;
	}

	/**
	 * @return the storageAccess
	 */
	public boolean isStorageAccess() {
		return storageAccess;
	}

	/**
	 * @param storageAccess the storageAccess to set
	 */
	public void setStorageAccess(boolean storageAccess) {
		this.storageAccess = storageAccess;
	}

	/**
	 * @return the location
	 */
	public GPSPosition getLocation() {
		return location;
	}

	/**
	 * @param location the location to set
	 */
	public void setLocation(GPSPosition location) {
		this.location = location;
	}

	public GieSQLite getDatabase() {
		return database;
	}

	public void setDatabase(GieSQLite database) {
		this.database = database;
	}

	public TreeMap<Integer, Select> getQueuedSelects() {
		return queuedSelects;
	}

	private void setQueuedSelects(TreeMap<Integer, Select> queuedSelects) {
		this.queuedSelects = queuedSelects;
	}

	public FavouritesSQLite getFavoritesDatabase() {
		return FavoritesDatabase;
	}

	public void setFavoritesDatabase(FavouritesSQLite favoritesDatabase) {
		FavoritesDatabase = favoritesDatabase;
	}

	public boolean isFromFavorites() {
		return fromFavorites;
	}

	public void setFromFavorites(boolean fromFavorites) {
		this.fromFavorites = fromFavorites;
	}
	
	public TreeMap<String, SinglePointObject> getGeofenceSPOMap() {
		return this.geofenceSPOMap;
	}
	
	/**
	 * returns the SinglePointObject from where the geofence was created
	 * @param geofenceId
	 * @return
	 */
	public SinglePointObject getSPO(String geofenceId) {
		return geofenceSPOMap.get(geofenceId);
	}

	public File getGieFolder() {
		return gieFolder;
	}

	public void setGieFolder(File gieFolder) {
		this.gieFolder = gieFolder;
	}

	public boolean isSoundNotifications() {
		return soundNotifications;
	}

	public void setSoundNotifications(boolean soundNotifications) {
		this.soundNotifications = soundNotifications;
	}

	public void startCoreService(Context context) {
		startService(new Intent(context, CoreService.class));
	}
	
	public void startMapActivity(Context context) {
		Intent mapIntent = new Intent(context, MapActivity.class);
		//starts the MapActivity and avoids the up navigation from here
		mapIntent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK|Intent.FLAG_ACTIVITY_NEW_TASK);
		startActivity(mapIntent);
	}
	
	/**
	 * Starts the CoreService and the MapActivity
	 * 
	 * @param context
	 */
	public void startCore(Context context) {
		startCoreService(context);
		startMapActivity(context);
	}
	
	public void broadcastMapChanging(Context context, boolean starting) {
		Intent i = new Intent("uoc.tfc.gien.MAP_CHANGING");
		if (starting) {
			i.putExtra("status", "start");
		} else {
			i.putExtra("status", "end");
		}
		context.sendBroadcast(i);
	}
	
	/**
	 * Starts LocationSenderIntentService
	 * @param context
	 */
	/*
	public void startLocationSender(Context context) {
		Intent intSrv = new Intent(context, uoc.tfc.gien.communication.SendLocationIntentService.class);
		startService(intSrv);
	}
	*/
	
	/**
	 * Loads GieInfo from json data
	 * 
	 * @param jsonInfo - json data
	 * @return true if the data is loaded, false if not
	 */
	public boolean loadGieInfo(JSONObject jsonInfo) {
		setGieInfo(new GieInfo());
		return gieInfo.loadJSON(jsonInfo);
	}
	
	/**
	 * Loads Experience from json data
	 * 
	 * @param jsonGie - json data
	 * @param isOnlineExp - indicates if the the experience is online or offline for proper load process
	 * @param context
	 * @return true if the data is loaded, false if not
	 */
	public boolean loadGie(JSONObject jsonGie, boolean isOnlineExp, Context context) {
		setExperience(new Experience());
		return gie.loadJSON(jsonGie, isOnlineExp, context);
	}
	
	
}
