/**
 * 
 */
package uoc.tfc.gien;

import java.io.File;

import org.json.JSONException;
import org.json.JSONObject;

import uoc.tfc.gien.R;
import uoc.tfc.gien.storage.Storage;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.res.Resources;
import android.content.Context;
import android.os.AsyncTask;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class LoadOfflineGie extends AsyncTask<File, Integer, Integer> {
	
	private ProgressDialog pd;
	private Context context;
	private Resources resources;
	final GlobalApplicationData app;
	private File folder;
	
	/**
	 * Constructor
	 * 
	 * @param context
	 */
	public LoadOfflineGie(Context context) {
		this.context = context;
		resources = context.getResources();
		app = (GlobalApplicationData) context.getApplicationContext();
	}
	
	@Override
    protected void onPreExecute() {
		pd = new ProgressDialog(context);
		pd.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
        pd.setIndeterminate(false);
        pd.setMessage(resources.getString(R.string.reading_gie_msg));
        pd.setTitle(resources.getString(R.string.please_wait));
        pd.setMax(100);
        pd.setProgress(0);
        pd.setCancelable(false);
        pd.setIcon(R.drawable.ic_sd_storage_blue);
        pd.show();
        super.onPreExecute();
    }
	
	protected Integer doInBackground(File... folder) {
		String infoJsonStr;
		String experienceJsonStr;
		boolean isInfoLoaded = false;
		boolean isGieLoaded = false;
		this.folder = folder[0];
		try {
			
			//read
			infoJsonStr = Storage.getInfoJsonContent(folder[0]);
			publishProgress(25);
			experienceJsonStr = Storage.getGieJsonContent(folder[0]);
			publishProgress(50);
			
			//loadgie
			JSONObject infoJsonObj = new JSONObject(infoJsonStr);
			isInfoLoaded = this.app.loadGieInfo(infoJsonObj.getJSONObject("info"));
			publishProgress(75);
			
			JSONObject gieJsonObj = new JSONObject(experienceJsonStr);
			isGieLoaded = this.app.loadGie(gieJsonObj.getJSONObject("experience"), false, context);
			publishProgress(100);

		} catch (JSONException e) {
			return -1; // parse error
		} catch (Exception e) {
			return -2; // I/O error
		}

		if ((isInfoLoaded)&&(isGieLoaded)) {
			this.app.setOnlineGie(false);
			return 0; //success
		} else {
			return -1; //parse error
		}
		
    }

    protected void onProgressUpdate(Integer... progress) {
    	super.onProgressUpdate(progress);
    	pd.setProgress(progress[0]);
    	if (progress[0]==50) {
    		pd.setMessage(resources.getString(R.string.loading_data_msg));
    	}
    }

    protected void onPostExecute(Integer status) {
    	pd.dismiss();
    	if (status<0) {
    		alertError(status);
    	} else {
    		app.setGieFolder(folder);
    		app.startCore(context);
    	}
    }
    
    public void alertError(int code) {
		AlertDialog.Builder dbuilder = new AlertDialog.Builder(context);
		dbuilder
			.setIconAttribute(android.R.attr.alertDialogIcon)
			.setTitle(R.string.error_capitals);
		switch (code) {
			case -1:
				dbuilder.setMessage(R.string.error_loading);
				break;
			case -2:
				dbuilder.setMessage(R.string.error_reading_files);
				break;
		}
		AlertDialog ad = dbuilder.create();
		ad.show();
	}
}
