/**
 * 
 */
package uoc.tfc.gien;


import java.net.URL;
import org.json.JSONException;
import org.json.JSONObject;

import uoc.tfc.gien.R;
import uoc.tfc.gien.communication.Communication;
import android.app.AlertDialog;

import android.app.ProgressDialog;
import android.content.res.Resources;
import android.content.Context;

import android.os.AsyncTask;
import android.widget.Toast;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class LoadOnlineGie extends AsyncTask<URL, Integer, Integer> {
	
	private ProgressDialog pd;
	private Context context;
	private Resources resources;
	final GlobalApplicationData app;
	
	/**
	 * Constructor
	 * 
	 * @param context
	 */
	public LoadOnlineGie(Context context) {
		this.context = context;
		resources = context.getResources();
		app = (GlobalApplicationData) context.getApplicationContext();
	}
	
	
	@Override
    protected void onPreExecute() {
		pd = new ProgressDialog(context);
		pd.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
        pd.setIndeterminate(false);
        pd.setMessage(resources.getString(R.string.downloading_gie_msg));
        pd.setTitle(resources.getString(R.string.please_wait));
        pd.setMax(100);
        pd.setProgress(0);
        pd.setCancelable(false);
        pd.setIcon(R.drawable.ic_cloud_download);
        pd.show();
        super.onPreExecute();
    }
	
	protected Integer doInBackground(URL... serverUrl) {
		String[] data = new String[2];
		boolean isInfoLoaded = false;
		boolean isGieLoaded = false;
		String serverUrlStr = null;
		try {
			
			//download
			if (serverUrl[0].getProtocol()==null) {
				serverUrlStr = "http://"+serverUrl[0].toString();
			} else {
				serverUrlStr = serverUrl[0].toString();
			}
			if (serverUrlStr.charAt(serverUrlStr.length()-1)!='/') {
				serverUrlStr = serverUrlStr+"/";
			}
			data[0] = Communication.downloadGieInfo(serverUrlStr);
			publishProgress(25);
			data[1] = Communication.downloadExperience(serverUrlStr);
			publishProgress(50);
			
			//loadgie
			JSONObject infoJsonObj = new JSONObject(data[0]);
			isInfoLoaded = this.app.loadGieInfo(infoJsonObj.getJSONObject("info"));
			publishProgress(75);
			
			JSONObject gieJsonObj = new JSONObject(data[1]);
			isGieLoaded = this.app.loadGie(gieJsonObj.getJSONObject("experience"),true, null);
			publishProgress(100);

		} catch (JSONException e) {
			return -1; //parse error
		} catch (Exception e) {
			return -2; //communication error
		}

		if ((isInfoLoaded)&&(isGieLoaded)) {
			this.app.setServerUrl(serverUrlStr);
			this.app.setOnlineGie(true);
			return 0; //success
		} else {
			return -1; //parse error
		}
		
    }

    protected void onProgressUpdate(Integer... progress) {
    	super.onProgressUpdate(progress);
    	pd.setProgress(progress[0]);
    	if (progress[0]==50) {
    		pd.setMessage(resources.getString(R.string.loading_data_msg));
    	}
    }

    protected void onPostExecute(Integer status) {
    	pd.dismiss();
    	if (status<0) {
    		alertError(status);
    	} else {
    		app.startCore(context);
    	}
    }
    
    public void alertError(int code) {
		AlertDialog.Builder dbuilder = new AlertDialog.Builder(context);
		dbuilder
			.setIconAttribute(android.R.attr.alertDialogIcon)
			.setTitle(R.string.error_capitals);
		switch (code) {
			case -1:
				dbuilder.setMessage(R.string.error_loading);
				break;
			case -2:
				dbuilder.setMessage(R.string.error_downloading);
				break;
		}
		AlertDialog ad = dbuilder.create();
		ad.show();
	}
}
