/**
 * 
 */
package uoc.tfc.gien;

import org.json.JSONException;
import org.json.JSONObject;

import uoc.tfc.gien.R;
import uoc.tfc.gien.communication.Communication;
import uoc.tfc.gien.entities.interactive.Select;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.res.Resources;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;


/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class TouchedLocationAsyncTask extends AsyncTask<Integer, Integer, Integer> {
	
	private ProgressDialog pd;
	private Context context;
	private Resources resources;
	final GlobalApplicationData app;
	private Select s;
	
	/**
	 * Constructor
	 * 
	 * @param context
	 */
	public TouchedLocationAsyncTask(Context context) {
		this.context = context;
		resources = context.getResources();
		app = (GlobalApplicationData) context.getApplicationContext();
	}
	
	
	@Override
    protected void onPreExecute() {
		pd = new ProgressDialog(context);
		pd.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        pd.setIndeterminate(false);
        pd.setMessage(resources.getString(R.string.retrieving_information_from_server));
        pd.setTitle(resources.getString(R.string.please_wait));
        pd.setCancelable(false);
        pd.setIcon(R.drawable.ic_cloud_download);
        pd.show();
        super.onPreExecute();
    }
	
	@Override
	protected Integer doInBackground(Integer... spoId) {
		String data = null;
		boolean isSelectLoaded = false;
		try {
			
			data = Communication.getSPOInformation(app.getServerUrl(), app.getLocation(), spoId[0]);
			s = new Select();
			isSelectLoaded = s.loadJSON(new JSONObject(data));

		} catch (JSONException e) {
			return -1; //parse error
		} catch (Exception e) {
			return -2; //communication error
		}

		if (isSelectLoaded) {
			return 0; //success
		} else {
			return -1; //parse error
		}
		
    }

    protected void onPostExecute(Integer status) {
    	pd.dismiss();
    	if (status<0) {
    		alertError(status);
    	} else {
			if (s!=null) {
				app.getQueuedSelects().put(s.getId(), s);
				Intent intent = new Intent(context, uoc.tfc.gien.ui.SelectActivity.class);
				intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
				intent.putExtra("id", s.getId());
				context.startActivity(intent);
			}
    	}
    }
    
    public void alertError(int code) {
		AlertDialog.Builder dbuilder = new AlertDialog.Builder(context);
		dbuilder
			.setIconAttribute(android.R.attr.alertDialogIcon)
			.setTitle(R.string.error_capitals);
		switch (code) {
			case -1:
				dbuilder.setMessage(R.string.error_loading);
				break;
			case -2:
				dbuilder.setMessage(R.string.error_downloading);
				break;
		}
		AlertDialog ad = dbuilder.create();
		ad.show();
	}

}
