/**
 * 
 */
package uoc.tfc.gien.communication;

import uoc.tfc.gien.GienException;
import uoc.tfc.gien.R;
import uoc.tfc.gien.entities.PairKV;
import uoc.tfc.gien.entities.geo.GPSPosition;

import java.util.ArrayList;
import java.util.List;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import java.net.URLConnection;

import android.content.res.Resources;
//import android.util.Log;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class Communication {
	
	final static String TAG = "Communication";
	private static Resources resources;
	
	/**
	 * Constructor
	 */
	public Communication() {
		// TODO Auto-generated constructor stub
	}
	
	public static void init(Resources resources) {
		setResources(resources);
	}
	
	private static void setResources(Resources res) {
		resources = res;
	}

	public static String getInvAndScore(GPSPosition pos, String serverUrl) {
		PairKV latitude = new PairKV("lat",String.valueOf(pos.getLatitude()));
		PairKV longitude = new PairKV("long",String.valueOf(pos.getLongitude()));
		ArrayList<PairKV> params = new ArrayList<PairKV>();
		params.add(latitude);
		params.add(longitude);
		String dataReceived = null;
		try {
			dataReceived = Communication.post(new URL(serverUrl+"INVENTORY_AND_SCORETABLE/"),params);
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}
		return dataReceived;
	}
	
	public static String getActions(GPSPosition pos, String serverUrl) {
		PairKV latitude = new PairKV("lat",String.valueOf(pos.getLatitude()));
		PairKV longitude = new PairKV("long",String.valueOf(pos.getLongitude()));
		ArrayList<PairKV> params = new ArrayList<PairKV>();
		params.add(latitude);
		params.add(longitude);
		String dataReceived = null;
		try {
			dataReceived = Communication.post(new URL(serverUrl+"ACTIONS/"),params);
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}
		return dataReceived;
	}
	
	public static String sendLocation(GPSPosition pos, String serverUrl) {
		PairKV latitude = new PairKV("lat",String.valueOf(pos.getLatitude()));
		PairKV longitude = new PairKV("long",String.valueOf(pos.getLongitude()));
		ArrayList<PairKV> params = new ArrayList<PairKV>();
		params.add(latitude);
		params.add(longitude);
		String dataReceived = null;
		try {
			dataReceived = Communication.post(new URL(serverUrl+"LOCATION/"),params);
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}
		return dataReceived;
	}
	
	public static String sendFunction(GPSPosition pos, String method, ArrayList<String> functionComponents) {
		ArrayList<PairKV> params = new ArrayList<PairKV>();
		PairKV latitude = new PairKV("lat",String.valueOf(pos.getLatitude()));
		PairKV longitude = new PairKV("long",String.valueOf(pos.getLongitude()));
		params.add(latitude);
		params.add(longitude);
		String dataReceived = null;
		for (int i=2; i<functionComponents.size();i++) {
			params.add(new PairKV("param"+String.valueOf(i-1),functionComponents.get(i)));
		}
		try {
			switch (RESTMethod.valueOf(method)) {
			case GET:
				dataReceived = Communication.get(new URL(functionComponents.get(1)), params);
				break;
			case POST:
				dataReceived = Communication.post(new URL(functionComponents.get(1)), params);
				break;
			}
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}
		return dataReceived;
	}
	
	public static String getSPOInformation(String serverUrl, GPSPosition pos, int spoId) {
		PairKV latitude = new PairKV("lat",String.valueOf(pos.getLatitude()));
		PairKV longitude = new PairKV("long",String.valueOf(pos.getLongitude()));
		PairKV spoid = new PairKV("sopid", String.valueOf(spoId));
		ArrayList<PairKV> params = new ArrayList<PairKV>();
		params.add(latitude);
		params.add(longitude);
		params.add(spoid);
		String dataReceived = null;
		try {
			dataReceived = Communication.post(new URL(serverUrl+"SELECTS/"),params);
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}
		return dataReceived;
	}
	
	/**
	 * Downloads gie.json from specified server
	 * 
	 * @param url
	 * @return downloaded text data
	 * @throws GienException
	 * @throws MalformedURLException
	 */
	public static String downloadExperience(String url) throws GienException, MalformedURLException {
		return get(new URL(url),null);
	}
	
	/**
	 * Downloads info.json from specified server
	 * 
	 * @param url
	 * @return downloaded text data
	 * @throws GienException
	 * @throws MalformedURLException
	 */
	public static String downloadGieInfo(String url) throws GienException, MalformedURLException {
		return get(new URL(url+"info.json"),null);
	}
	
	private static String getQuery(List<PairKV> params) throws UnsupportedEncodingException
	{
	    StringBuilder result = new StringBuilder();
	    boolean first = true;

	    for (PairKV pair : params)
	    {
	        if (first)
	            first = false;
	        else
	            result.append("&");

	        //result.append(URLEncoder.encode(pair.getKey(), "UTF-8"));
	        result.append(pair.getKey());
	        result.append("=");
	        //result.append(URLEncoder.encode(pair.getValue(), "UTF-8"));
	        result.append(pair.getValue());
	    }

	    return result.toString();
	}
	
	private static HttpURLConnection makeConnection(RESTMethod method, URL url, List<PairKV> params) throws GienException {
		HttpURLConnection urlConnection = null;
		int connectTimeout = 10*1000;
		int readTimeout = 15*1000;
		PairKV propertyAgent = new PairKV("User-Agent", "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)");
		try {
			switch (method) {
				case POST:
					urlConnection = (HttpURLConnection) url.openConnection();
					urlConnection.setDoOutput(true);
					
					if (!params.isEmpty()) {
						OutputStream os = urlConnection.getOutputStream();
						BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(os, "UTF-8"));
						writer.write(getQuery(params));
						writer.flush();
						writer.close();
						os.close();
					}
					break;
				case GET:
					if(params!=null) {
						StringBuilder urlGet = new StringBuilder();
						urlGet
							.append(url.toString())
							.append("?")
							.append(getQuery(params));
						urlConnection = (HttpURLConnection) (new URL(urlGet.toString())).openConnection();
					} else {
						urlConnection = (HttpURLConnection) url.openConnection();
					}
					urlConnection.setRequestMethod(method.name());
					urlConnection.setConnectTimeout(connectTimeout);
					urlConnection.setReadTimeout(readTimeout);
					break;
				case PUT:
					break;
				case DELETE:
					break;
				case HEAD:
					break;
			}
			
			return urlConnection;
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
			throw (new GienException(resources.getString(R.string.exception_connection_error)));
		} finally {
			if (urlConnection!=null) {
				urlConnection.disconnect();
			}
		}
	}
	
	public static String sendRESTOperation(RESTMethod method, URL url, List<PairKV> params) throws GienException {
		StringBuilder dataFromServer = new StringBuilder();
		BufferedReader reader = null;
		HttpURLConnection urlConnection = null;
		try {
			urlConnection = makeConnection(method, url, params);
			if (urlConnection.getResponseCode()==HttpURLConnection.HTTP_OK) {
				reader = new BufferedReader(new InputStreamReader(urlConnection.getInputStream(), "UTF-8"));
				String linea = reader.readLine();

				while (linea!=null) {
					dataFromServer.append(linea);
					linea = reader.readLine();
				}
			}
			else {
				throw new GienException(resources.getString(R.string.bad_server_response)+urlConnection.getResponseMessage());
			}
		
		} catch (IOException e) {
			throw new GienException(resources.getString(R.string.exception_cant_retrieve_data));
		} finally {
			if (reader!=null) {
				try {
					reader.close();
				} catch (IOException e) {
					//Log.w(TAG,e.getMessage());
				}
			}
			if (urlConnection != null) {
				urlConnection.disconnect();
			}
		}
		return dataFromServer.toString();
	}
	
	/**
	 * Sends an HTTP GET request and returns text data
	 * 
	 * @param url
	 * @param params 
	 * @return response from the server resulting of the petition
	 * 
	 * @throws GienException
	 */
	public static String get(URL url, List<PairKV> params) throws GienException {
		return sendRESTOperation(RESTMethod.GET, url, params);
	}
	
	/**
	 * Sends an HTTP POST request and returns text data
	 * 
	 * @param url
	 * @param params 
	 * @return response from the server resulting of the petition
	 * 
	 * @throws GienException
	 */
	public static String post(URL url, List<PairKV> params) throws GienException {
		StringBuilder dataFromServer = new StringBuilder();
		BufferedReader reader = null;
		URLConnection urlConnection = null;
		HttpURLConnection hucon = null;
		
		OutputStream os = null;
		BufferedWriter writer = null;
		
		try {
			urlConnection = url.openConnection();
			urlConnection.setDoOutput(true);
			urlConnection.connect();
						
			if (!params.isEmpty()) {
				os = urlConnection.getOutputStream();
				writer = new BufferedWriter(new OutputStreamWriter(os, "UTF-8"));
				writer.write(getQuery(params));
				writer.flush();
				writer.close();
				os.close();
			}
			
			hucon = (HttpURLConnection)urlConnection;
			if (hucon.getResponseCode()==HttpURLConnection.HTTP_OK) {
				reader = new BufferedReader(new InputStreamReader(hucon.getInputStream(), "UTF-8"));
				String linea = reader.readLine();

				while (linea!=null) {
					dataFromServer.append(linea);
					linea = reader.readLine();
				}
				reader.close();
			}
			else {
				throw new GienException(resources.getString(R.string.bad_server_response)+hucon.getResponseMessage());
			}
			
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		} finally {
			if (urlConnection != null) {
				hucon.disconnect();
			}
			try {
				if (reader!=null) {
					reader.close();
				}
				if (writer!=null) {
					writer.close();
				}		
			} catch (IOException e) {
				//Log.e(TAG,e.getMessage());
			}
		}
		return dataFromServer.toString();	
		
	}
	
}
