/**
 * 
 */
package uoc.tfc.gien.database;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
//import android.util.Log;

import java.net.URL;
import java.util.Vector;

import uoc.tfc.gien.GienException;
import uoc.tfc.gien.R;
import uoc.tfc.gien.entities.Favorite;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class FavouritesSQLite extends SQLiteOpenHelper {
	
	final String TAG = "FavouritesSQLite";
	private static FavouritesSQLite instance;
	private static Context context;

	/**
	 * Constructor
	 * 
	 * @param context
	 */
	public FavouritesSQLite(Context cntxt) {
		super(cntxt, "favourites", null, 1);
		context = cntxt;
	}
	
	public static synchronized FavouritesSQLite getInstance(Context cntxt) {
		if (instance==null) {
			instance = new FavouritesSQLite(cntxt.getApplicationContext());
		}
		return instance;
	}
	
	/* (non-Javadoc)
	 * @see android.database.sqlite.SQLiteOpenHelper#onCreate(android.database.sqlite.SQLiteDatabase)
	 */
	@Override
	public void onCreate(SQLiteDatabase db) {
		try {
			db.execSQL("CREATE TABLE favourites ( "+
								"id INTEGER PRIMARY KEY AUTOINCREMENT, "+
								"url TEXT NOT NULL UNIQUE, "+
								"username TEXT, "+
								"password TEXT, "+
								"name TEXT NOT NULL)");
			
			db.execSQL("INSERT INTO favourites VALUES ( null, 'http://www.worldnected.com/tfc/test6', null, null,"+
					" 'test6')");
			
			
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}
	}

	/* (non-Javadoc)
	 * @see android.database.sqlite.SQLiteOpenHelper#onUpgrade(android.database.sqlite.SQLiteDatabase, int, int)
	 */
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		// in case of a new database version we need to upgrade the tables
		// upgrade code here
	}
	
	/**
	 * Adds a new favourite to the database
	 * 
	 * @param url
	 * @param username
	 * @param password
	 * @param name
	 */
	public void addReg(String url, String username, String password, String name) throws GienException {
		SQLiteDatabase db;
		try {
			db = getWritableDatabase();
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
			throw new GienException(context.getString(R.string.error_opening_database));
		}
		ContentValues cv = new ContentValues();
		cv.put("url", url);
		cv.put("username", username);
		cv.put("password", password);
		cv.put("name", name);
		if (db.insert("favourites", "password", cv) == -1) {
			throw new GienException(context.getString(R.string.error_saving_favorite));
		}
	}
	
	/**
	 * Returns a list of all the favourites stored in the database
	 * 
	 * @return list of all the favourites in the database
	 */
	public Vector<Favorite> getListReg() {
		Vector<Favorite> result = new Vector<Favorite>();
		SQLiteDatabase db = getReadableDatabase();
		Cursor cursor = db.rawQuery("SELECT * FROM favourites ORDER BY name ASC", null);
		while (cursor.moveToNext()) {
			try {
			result.add(new Favorite(cursor.getInt(0),
									new URL(cursor.getString(1)),
									cursor.getString(2),
									cursor.getString(3),
									cursor.getString(4)));
			}
			catch (Exception e){
				//Log.e(TAG,e.getMessage());
			}
		}
		cursor.close();
		return result;
	}

	/**
	 * Updates the data of the register identified with id
	 * 
	 * @param id
	 * @param url
	 * @param username
	 * @param password
	 * @param name
	 */
	public void updateReg(int id, String url, String username, String password, String name) throws GienException {
		SQLiteDatabase db;
		try {
			db = getWritableDatabase();
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
			throw new GienException(context.getString(R.string.error_opening_database));
		}
		ContentValues cv = new ContentValues();
		cv.put("url",url);
		cv.put("username",username);
		cv.put("password", password);
		cv.put("name", name);
		if (db.update("favourites", cv, "id="+id, null)==0) {
			throw new GienException(context.getString(R.string.error_updating_favorite));
		}
	}
	
	/**
	 * Deletes the register identified with the specified id
	 * 
	 * @param id
	 */
	public void deleteReg(int id) throws GienException {
		SQLiteDatabase db;
		try {
			db = getWritableDatabase();
			db.execSQL("DELETE FROM favourites WHERE id="+id);
		} catch (SQLiteException e) {
			//Log.e(TAG,e.getMessage());
			throw new GienException(context.getString(R.string.error_opening_database));
		} catch (SQLException e) {
			//Log.e(TAG,e.getMessage());
			throw new GienException(context.getString(R.string.error_deleting_favorite));
		}
	}
}
