/**
 * 
 */
package uoc.tfc.gien.database;

import java.util.ArrayList;
import java.util.Iterator;

import uoc.tfc.gien.GienException;
import uoc.tfc.gien.R;
import uoc.tfc.gien.entities.Experience;
import uoc.tfc.gien.entities.interactive.Message;
import uoc.tfc.gien.entities.interactive.Option;
import uoc.tfc.gien.entities.interactive.Resource;
import uoc.tfc.gien.entities.interactive.Select;
import uoc.tfc.gien.entities.interactive.Usable;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteStatement;
//import android.util.Log;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class GieSQLite extends SQLiteOpenHelper {
	
	final String TAG = "GieSQLite";
	private static GieSQLite instance;
	private Context context;

	/**
	 * 
	 * @param context
	 */
	private GieSQLite(Context context) {
		super(context, "gie", null, 1);
		this.context = context;
	}
	
	/**
	 * 
	 * @param context
	 * @return
	 */
	public static synchronized GieSQLite getInstance(Context context) {
		if (instance == null) {
			instance = new GieSQLite(context.getApplicationContext());
		}
		return instance;
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
		StringBuilder createMessagesTable = new StringBuilder();
		createMessagesTable
			.append("CREATE TABLE messages ( ")
			.append("id INTEGER PRIMARY KEY, ")
			.append("title TEXT NOT NULL, ")
			.append("text TEXT NOT NULL, ")
			.append("type TEXT NOT NULL)"); //plaintext or html

		StringBuilder createSelectsTable = new StringBuilder();
		createSelectsTable
			.append("CREATE TABLE selects ( ")
			.append("id INTEGER PRIMARY KEY, ")
			.append("title TEXT NOT NULL, ")
			.append("description TEXT NOT NULL)");
		
		StringBuilder createResourcesTable = new StringBuilder();
		createResourcesTable
			.append("CREATE TABLE resources ( ")
			.append("id INTEGER PRIMARY KEY, ")
			.append("name TEXT NOT NULL, ")
			.append("description TEXT, ")
			.append("uri TEXT, ")
			.append("localpath TEXT, ") //relative to filetype folder
			.append("type TEXT NOT NULL)");
		
		StringBuilder createOptionsTable = new StringBuilder();
		createOptionsTable
			.append("CREATE TABLE options ( ")
			.append("id INTEGER PRIMARY KEY, ")
			.append("name TEXT NOT NULL, ")
			.append("description TEXT NOT NULL, ")
			.append("function TEXT NOT NULL)");
		
		StringBuilder createItemsInSelectTable = new StringBuilder();
		createItemsInSelectTable
			.append("CREATE TABLE itemsinselect ( ")
			.append("idforeign INTEGER NOT NULL, ")
			.append("idselect INTEGER NOT NULL, ")
			.append("entitytype TEXT NOT NULL, ")
			.append("PRIMARY KEY (idforeign, idselect, entitytype))");
		
		db.beginTransaction();
		try {
			db.execSQL(createMessagesTable.toString());
			db.execSQL(createSelectsTable.toString());
			db.execSQL(createResourcesTable.toString());
			db.execSQL(createOptionsTable.toString());
			db.execSQL(createItemsInSelectTable.toString());
			db.setTransactionSuccessful();
		} catch (Exception e) {
			//Log.e("gien-GieSQLite-onCreate",e.getMessage());
		} finally {
			db.endTransaction();
		}
			
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		// TODO Auto-generated method stub

	}
	
	public void insertResources(ArrayList<Resource> list) {
		SQLiteDatabase db = getWritableDatabase();
		String sql = "INSERT INTO resources VALUES (?,?,?,?,?,?)";
		db.beginTransaction();
		try {
			SQLiteStatement statement = db.compileStatement(sql);
			Iterator<Resource> it = list.iterator();
			while (it.hasNext()) {
				Resource obj = it.next();
				statement.clearBindings();
                statement.bindLong(1, obj.getId());
                statement.bindString(2, obj.getName());
                statement.bindString(3, obj.getDescription());
                statement.bindString(4, obj.getUri().toString());
                statement.bindString(5, obj.getLocalFilePath()); //relative to filetype folder
                statement.bindString(6, obj.getContenttype().name());
                statement.execute();
			}
			db.setTransactionSuccessful();
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		} finally {
			db.endTransaction();
		}
	}
	
	public void insertMessages(ArrayList<Message> list) {
		SQLiteDatabase db = getWritableDatabase();
		String sql = "INSERT INTO messages VALUES (?,?,?,?)";
		db.beginTransaction();
		try {
			SQLiteStatement statement = db.compileStatement(sql);
			Iterator<Message> it = list.iterator();
			while (it.hasNext()) {
				Message obj = it.next();
				statement.clearBindings();
                statement.bindLong(1, obj.getId());
                statement.bindString(2, obj.getTitle());
                statement.bindString(3, obj.getText());
                statement.bindString(4, obj.getTextType().name());
                statement.execute();
			}
			db.setTransactionSuccessful();
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		} finally {
			db.endTransaction();
		}
	}
	
	public void insertSelects(ArrayList<Select> list) {
		SQLiteDatabase db = getWritableDatabase();
		String sql = "INSERT INTO selects VALUES (?,?,?)";
		String sqlIIS = "INSERT INTO itemsinselect VALUES (?,?,?)";
        String sqlO = "INSERT INTO options VALUES (?,?,?,?)";
        String sqlR = "INSERT INTO resources VALUES (?,?,?,?,?,?)";
        
        Usable.Type entitytype;
        int idselect = 0;
        int idforeign = 0;
        
		db.beginTransaction();
		try {
			SQLiteStatement statement = db.compileStatement(sql);
			SQLiteStatement statementIIS = db.compileStatement(sqlIIS);
            SQLiteStatement statementO = db.compileStatement(sqlO);
            SQLiteStatement statementR = db.compileStatement(sqlR);
            
			Iterator<Select> it = list.iterator();
			while (it.hasNext()) {
				Select obj = it.next();
				statement.clearBindings();
				idselect = obj.getId();
                statement.bindLong(1, idselect);
                statement.bindString(2, obj.getTitle());
                statement.bindString(3, obj.getText());
                statement.execute();
                
                ArrayList<Usable> items = obj.getItems();                
                Iterator<Usable> itU = items.iterator();
                while(itU.hasNext()) {
                	entitytype = null;
                	Usable us = (Usable)itU.next();
                	db.beginTransaction();
                	try {
                		if (us instanceof Option) {
                    		statementO.clearBindings();
                    		idforeign = ((Option)us).getId();
                            statementO.bindLong(1, idforeign);
                            statementO.bindString(2, ((Option)us).getName());
                            statementO.bindString(3, ((Option)us).getDescription());
                            statementO.bindString(4, ((Option)us).getFunction());
                            statementO.execute();
                            entitytype = Usable.Type.OPTION;
                    	} else {
                    		if (us instanceof Resource) {
                    			statementR.clearBindings();
                    			idforeign = ((Resource)us).getId();
                                statementR.bindLong(1, idforeign);
                                statementR.bindString(2, ((Resource)us).getName());
                                statementR.bindString(3, ((Resource)us).getDescription());
                                //statementR.bindString(4, ((Resource)us).getUri().toString());
                                statementR.bindString(4, " ");
                                statementR.bindString(5, ((Resource)us).getLocalFilePath()); //relative to filetype folder
                                statementR.bindString(6, ((Resource)us).getContenttype().name());
                                statementR.execute();
                                entitytype = Usable.Type.RESOURCE;
                    		}
                    	}
                		if (entitytype==null) {
                			throw new GienException(context.getString(R.string.invalid_usable_type));
                		}
                		statementIIS.clearBindings();
                		statementIIS.bindLong(1,idforeign);
                		statementIIS.bindLong(2,idselect);
                		statementIIS.bindString(3,entitytype.name());
                		statementIIS.execute();
                		db.setTransactionSuccessful();
                		db.endTransaction();
                	} catch (Exception e){
                		db.endTransaction();
                	}
                }
                
			}
			db.setTransactionSuccessful();
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		} finally {
			db.endTransaction();
		}
	}
	
	public void insertOptions(ArrayList<Option> list) {
		SQLiteDatabase db = getWritableDatabase();
		String sql = "INSERT INTO options VALUES (?,?,?,?)";
		db.beginTransaction();
		try {
			SQLiteStatement statement = db.compileStatement(sql);
			Iterator<Option> it = list.iterator();
			while (it.hasNext()) {
				Option obj = it.next();
				statement.clearBindings();
                statement.bindLong(1, obj.getId());
                statement.bindString(2, obj.getName());
                statement.bindString(3, obj.getDescription());
                statement.bindString(4, obj.getFunction());
                statement.execute();
			}
			db.setTransactionSuccessful();
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		} finally {
			db.endTransaction();
		}
	}
	
	private void insertItemsInSelect(ArrayList<ItemInSelect> list) {
		SQLiteDatabase db = getWritableDatabase();
		String sql = "INSERT INTO itemsinselect VALUES (?,?,?)";
		db.beginTransaction();
		try {
			SQLiteStatement statement = db.compileStatement(sql);
			Iterator<ItemInSelect> it = list.iterator();
			while (it.hasNext()) {
				ItemInSelect obj = it.next();
				statement.clearBindings();
                statement.bindLong(1, obj.getIdForeign());
                statement.bindLong(2, obj.getIdSelect());
                statement.bindString(3, obj.getType());
                statement.execute();
			}
			db.setTransactionSuccessful();
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		} finally {
			db.endTransaction();
		}
	}
	
	public Message getMessage(int id) {
		if (id==0) return null;
		Message m = null;
		try {
			SQLiteDatabase db = getWritableDatabase();
			String sql = "SELECT * FROM messages WHERE id="+id;
			Cursor cursor = db.rawQuery(sql, null);
			if (cursor!=null) {
				if (cursor.moveToFirst()) {
					m = new Message();
					m.setId(cursor.getInt(0));
					m.setTitle(cursor.getString(1));
					m.setText(cursor.getString(2));
					m.setTextType(Message.TextType.valueOf(cursor.getString(3)));
				}
			}
			return m;
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
			return null;
		}
	}
	
	public Resource getResource(int id) {
		if (id==0) return null;
		Resource r = null;
		try {
			SQLiteDatabase db = getWritableDatabase();
			String sql = "SELECT * FROM resources WHERE id="+id;
			Cursor cursor = db.rawQuery(sql, null);
			if (cursor!=null) {
				if (cursor.moveToFirst()) {
					r = resourceFromCursor(cursor);
				}
			}
			return r;
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
			return null;
		}
	}
	
	
	private Resource resourceFromCursor(Cursor c) {
		if (c!=null) {
			Resource r = new Resource();
			r.setId(c.getInt(0));
			r.setName(c.getString(1));
			if (!c.isNull(2)) {
				r.setDescription(c.getString(2));
			}
			if (!c.isNull(3)) {
				r.setUri(c.getString(3));
			}
			if (!c.isNull(4)) {
				r.setLocalFilePath(c.getString(4));
			}
			r.setContenttype(Resource.ContentType.valueOf(c.getString(5)));
			return r;
		}
		else return null;
	}

	
	public Select getSelect(int id) {
		if (id==0) return null;
		Select s = null;
		try {
			SQLiteDatabase db = getWritableDatabase();
			String sqlS = "SELECT * FROM selects WHERE id="+id;
			Cursor cursorS = db.rawQuery(sqlS, null);
			if (cursorS!=null) {
				if (cursorS.moveToFirst()) {
					s = new Select();
					s.setId(cursorS.getInt(0));
					s.setTitle(cursorS.getString(1));
					s.setText(cursorS.getString(2));
					
					//get items
					ArrayList<Usable> items = new ArrayList<Usable>();
					
					// get resources
					String sqlIt = "SELECT * FROM resources WHERE id IN (SELECT idforeign FROM itemsinselect WHERE idselect="+id+" AND entitytype='RESOURCE')";
					Cursor cursorIt = db.rawQuery(sqlIt, null);
					if (cursorIt!=null) {
						if(cursorIt.moveToFirst()) {
							Resource r = null;
							do {
								r = resourceFromCursor(cursorIt);
								if (r!=null) {
									items.add(r);
								}
							} while (cursorIt.moveToNext());
						}
					}
					
					// get options
					sqlIt = "SELECT * FROM options WHERE id IN (SELECT idforeign FROM itemsinselect WHERE idselect="+id+" AND entitytype='OPTION')";
					cursorIt = db.rawQuery(sqlIt, null);
					if (cursorIt!=null) {
						if(cursorIt.moveToFirst()) {
							Option o = null;
							do {
								o = new Option();
								o.setId(cursorIt.getInt(0));
								o.setName(cursorIt.getString(1));
								o.setDescription(cursorIt.getString(2));
								o.setFunction(cursorIt.getString(3));
								items.add(o);
							} while (cursorIt.moveToNext());
						}
					}
					s.setItems(items);
				}
			}
			return s;
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
			return null;
		}
	}

	public void clean() {
		SQLiteDatabase db = getWritableDatabase();
		db.beginTransaction();
		try {
			db.delete("messages", null, null);
			db.delete("resources", null, null);
			db.delete("selects", null, null);
			db.delete("options", null, null);
			db.delete("itemsinselect", null, null);
			db.setTransactionSuccessful();
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		} finally {
			db.endTransaction();
		}
	}

}
