/**
 * 
 */
package uoc.tfc.gien.entities;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.json.JSONObject;


/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class GieInfo {
	
	private String name;							// name of the gie
	private String description;						// description of the gie
	private int version;							// identifies the version of the gie
	private Date date;								// date when the gie was last modified
	private int fileFormatVersion;					// identifies the version of the format specification
	private String url;								// url of the root folder of the gie
	private String author;							// name of the author of the gie
	private String urlWeb;							// url of the official website of this gie
	private String email;							// contact email
	
	
	/**
	 * Constructor
	 */
	public GieInfo() {
		//TO DO
	}
	
	/**
	 * Constructor
	 * 
	 * @param name
	 * @param description
	 * @param version
	 * @param date
	 * @param fileFormatVersion
	 * @param url
	 * @param author
	 * @param urlWeb
	 * @param email
	 */
	public GieInfo(String name, String description, int version, Date date, int fileFormatVersion,
				String url, String author, String urlWeb, String email) {
		this.setName(name);
		this.setDescription(description);
		this.setVersion(version);
		this.setDate(date);
		this.setFileFormatVersion(fileFormatVersion);
		this.setUrl(url);
		this.setAuthor(author);
		this.setUrlWeb(urlWeb);
		this.setEmail(email);
	}
	
	/**
	 * Loads the object properties from a JSONObject
	 * 
	 * @param object
	 * @return true if success, false if not
	 */
	public boolean loadJSON(JSONObject object) {
		try {
			this.setName(object.getString("name"));
			this.setDescription(object.getString("description"));
			this.setVersion(object.getInt("version"));
			SimpleDateFormat df = new SimpleDateFormat("dd-MM-yyyy");
			this.setDate(df.parse(object.getString("date")));
			this.setFileFormatVersion(object.getInt("file_format_version"));
			this.setUrl(object.getString("url"));
			this.setAuthor(object.getString("author"));
			this.setUrlWeb(object.getString("web"));
			this.setEmail(object.getString("email"));
		} catch (Exception e) {
			//Log.e("gien-GieInfo-loadJSON",e.getMessage());
			return false;
		}
		return true;
	}
	
	public static String[] getNameAndDescription(String content) {
		String[] ret = new String[2];
		try {
			JSONObject object = new JSONObject(content);
			JSONObject info = object.getJSONObject("info"); 
			ret[0] = info.getString("name");
			ret[1] = info.getString("description");
		} catch (Exception e) {
			//Log.d("gien-GieInfo-getNameAndDescription",e.getMessage());
		}
		return ret;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the version
	 */
	public int getVersion() {
		return version;
	}

	/**
	 * @param version the version to set
	 */
	public void setVersion(int version) {
		this.version = version;
	}

	/**
	 * @return the date
	 */
	public Date getDate() {
		return date;
	}

	/**
	 * @param date the date to set
	 */
	public void setDate(Date date) {
		this.date = date;
	}

	/**
	 * @return the fileFormatVersion
	 */
	public int getFileFormatVersion() {
		return fileFormatVersion;
	}

	/**
	 * @param fileFormatVersion the fileFormatVersion to set
	 */
	public void setFileFormatVersion(int fileFormatVersion) {
		this.fileFormatVersion = fileFormatVersion;
	}

	/**
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * @param url the url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}

	/**
	 * @return the author
	 */
	public String getAuthor() {
		return author;
	}

	/**
	 * @param author the author to set
	 */
	public void setAuthor(String author) {
		this.author = author;
	}

	/**
	 * @return the urlWeb
	 */
	public String getUrlWeb() {
		return urlWeb;
	}

	/**
	 * @param urlWeb the urlWeb to set
	 */
	public void setUrlWeb(String urlWeb) {
		this.urlWeb = urlWeb;
	}

	/**
	 * @return the email
	 */
	public String getEmail() {
		return email;
	}

	/**
	 * @param email the email to set
	 */
	public void setEmail(String email) {
		this.email = email;
	}
	
	public String getDateStr() {
		SimpleDateFormat df = new SimpleDateFormat("dd-MM-yyyy");
		return df.format(getDate());
	}
		
}
