/**
 *
 */
package uoc.tfc.gien.entities.geo;

import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONObject;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class Layer {

	private ArrayList<GeoObject> objects;
	private boolean visibility;
	private int zindex;
	private int id;
	private String name;
	private String description;

	/**
	 * Constructor
	 */
	public Layer() {
		this.objects = new ArrayList<GeoObject>();
	}
	
	/**
	 * Constructor
	 * 
	 * @param id
	 * @param name
	 * @param description
	 * @param zindex
	 * @param visibility
	 * @param objects
	 */
	public Layer(int id, String name, String description, int zindex, boolean visibility, ArrayList<GeoObject> objects) {
		this.setId(id);
		this.setName(name);
		this.setDescription(description);
		this.setZindex(zindex);
		this.setVisibility(visibility);
		this.setObjects(objects);
		if (this.getObjects()==null) {
			this.setObjects(new ArrayList<GeoObject>());
		}
	}

	/**
	 * @return the visibility
	 */
	public boolean isVisible() {
		return visibility;
	}

	/**
	 * @param visibility the visibility to set
	 */
	public void setVisibility(boolean visibility) {
		this.visibility = visibility;
	}

	/**
	 * @return the zindex
	 */
	public int getZindex() {
		return zindex;
	}

	/**
	 * @param zindex the zindex to set
	 */
	public void setZindex(int zindex) {
		this.zindex = zindex;
	}

	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the objects
	 */
	public ArrayList<GeoObject> getObjects() {
		return objects;
	}

	/**
	 * @param objects the objects to set
	 */
	public void setObjects(ArrayList<GeoObject> objects) {
		this.objects = objects;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}
	
	/**
	 * Loads the object properties from a JSONObject
	 * 
	 * @param object
	 * @return true if success, false if not
	 */
	public boolean loadJSON(JSONObject object) {
		try {
			this.setId(object.getInt("id"));
			this.setName(object.getString("name"));
			this.setDescription(object.getString("description"));
			this.setZindex(object.getInt("zindex"));
			this.setVisibility(object.getBoolean("visibility"));
			JSONArray jsonGeoObjects = object.getJSONArray("geoObject");
			if (jsonGeoObjects.length()>0) {
				for(int i=0; i<jsonGeoObjects.length();i++) {
					GeoObject gObject = null;
					String type = jsonGeoObjects.getJSONObject(i).getString("type");
					switch (GeoObject.Type.valueOf(type)) {
						case SINGLE_POINT:
							gObject = new SinglePointObject();
							break;
						case MULTI_POINT:
							gObject = new MultiPointObject();
							break;
						case ROUTE:
							gObject = new Route();
							break;
						case POLYGON:
							gObject = new Polygon();
							break;
					}
					if (gObject!=null) {
						gObject.loadJSON(jsonGeoObjects.getJSONObject(i));
						this.objects.add(gObject);
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

}
