/**
 *
 */
package uoc.tfc.gien.entities.geo;

import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONObject;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class Map {

	private int id;
	private String name;
	private String description;
	private ArrayList<Layer> layers;
	private GPSPosition origin; //top-left corner of the rectangle
	private GPSPosition end; //bottom-right corner of the rectangle

	/**
	 * Constructor
	 */
	public Map() {
		this.setLayers(new ArrayList<Layer>());
	}
	
	/**
	 * Constructor
	 * 
	 * @param id
	 * @param name
	 * @param description
	 * @param layers
	 * @param origin
	 * @param end
	 */
	public Map(int id, String name, String description, ArrayList<Layer> layers, GPSPosition origin, GPSPosition end) {
		this.setId(id);
		this.setName(name);
		this.setDescription(description);
		this.setLayers(layers);
		if (this.getLayers()==null) {
			this.setLayers(new ArrayList<Layer>());
		}
		this.setOrigin(origin);
		this.setEnd(end);
	}

	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the layers
	 */
	public ArrayList<Layer> getLayers() {
		return layers;
	}

	/**
	 * @param layers the layers to set
	 */
	public void setLayers(ArrayList<Layer> layers) {
		this.layers = layers;
	}

	/**
	 * @return the origin
	 */
	public GPSPosition getOrigin() {
		return origin;
	}

	/**
	 * @param origin the origin to set
	 */
	public void setOrigin(GPSPosition origin) {
		this.origin = origin;
	}

	/**
	 * @return the end
	 */
	public GPSPosition getEnd() {
		return end;
	}

	/**
	 * @param end the end to set
	 */
	public void setEnd(GPSPosition end) {
		this.end = end;
	}
	
	/**
	 * Loads the object properties from a JSONObject
	 * 
	 * @param object
	 * @return true if success, false if not
	 */
	public boolean loadJSON(JSONObject object) {
		try {
			this.setId(object.getInt("id"));
			this.setName(object.getString("name"));
			this.setDescription(object.getString("description"));
			JSONArray jsonLayers = object.getJSONArray("layer");
			if (jsonLayers.length()>0) {
				for(int i=0; i<jsonLayers.length();i++) {
					Layer l = new Layer();
					l.loadJSON(jsonLayers.getJSONObject(i));
					this.layers.add(l);
				}
			}
			GPSPosition jsonOrigin = new GPSPosition();
			jsonOrigin.loadJSON(object.getJSONObject("origin"));
			this.setOrigin(jsonOrigin);
			GPSPosition jsonEnd = new GPSPosition();
			jsonEnd.loadJSON(object.getJSONObject("end"));
			this.setEnd(jsonEnd);
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

}
