/**
 * 
 */
package uoc.tfc.gien.entities.geo;

import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONObject;


/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class MultiPointObject extends GeoObject {

	private ArrayList<SinglePointObject> points;
	
	/**
	 * Constructor
	 */
	public MultiPointObject() {
		super();
		this.setType(Type.MULTI_POINT);
		this.points = new ArrayList<SinglePointObject>();
	}

	/**
	 * Constructor
	 * 
	 * @param id
	 * @param name
	 * @param description
	 * @param visibility
	 * @param select
	 * @param visited
	 * @param points
	 */
	public MultiPointObject(int id, String name, String description,
			boolean visibility, int selectId, boolean visited, ArrayList<SinglePointObject> points) {
		super(id, name, description, visibility, selectId, visited);
		this.setType(Type.MULTI_POINT);
		this.setPoints(points);
		if (this.getPoints()==null) {
			this.setPoints(new ArrayList<SinglePointObject>());
		}
	}
	
	/**
	 * Loads the object properties from a JSONObject
	 * 
	 * @param object
	 * @return true if success, false if not
	 */
	public boolean loadJSON(JSONObject object) {
		super.loadJSON(object);
		try {
			JSONArray jsonSPArray = object.getJSONArray("singlepoint");
			if (jsonSPArray.length()>0) {
				for(int i=0; i<jsonSPArray.length();i++) {
					SinglePointObject p = new SinglePointObject();
					p.loadJSON(jsonSPArray.getJSONObject(i));
					this.points.add(p);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	/**
	 * @return the points
	 */
	public ArrayList<SinglePointObject> getPoints() {
		return points;
	}

	/**
	 * @param points the points to set
	 */
	public void setPoints(ArrayList<SinglePointObject> points) {
		this.points = points;
	}

}
