/**
 * 
 */
package uoc.tfc.gien.entities.geo;

import org.json.JSONObject;


/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class SinglePointObject extends GeoObject {

	private GPSPosition center;
	private double actionRadiusMeters;
	private float color; // color del marcador (valor HUE, rang [0,360])
	
	/**
	 * Constructor
	 */
	public SinglePointObject() {
		super();
		this.setType(Type.SINGLE_POINT);
	}

	/**
	 * Constructor
	 * 
	 * @param id
	 * @param name
	 * @param description
	 * @param visibility
	 * @param select
	 * @param visited
	 * @param center
	 * @param radius
	 * @param color
	 */
	public SinglePointObject(int id, String name, String description, boolean visibility, int selectId, boolean visited,
			GPSPosition center, double radius, float color) {
		super(id, name, description, visibility, selectId, visited);
		this.setCenter(center);
		this.setActionRadiusMeters(radius);
		this.setType(Type.SINGLE_POINT);
		this.setColor(color);
	}
	
	/**
	 * Loads the object properties from a JSONObject
	 * 
	 * @param object
	 * @return true if success, false if not
	 */
	public boolean loadJSON(JSONObject object) {
		super.loadJSON(object);
		try {
			GPSPosition jsonCenter = new GPSPosition();
			jsonCenter.loadJSON(object.getJSONObject("center"));
			this.setCenter(jsonCenter);
			this.setActionRadiusMeters(object.getDouble("radius"));
			this.setColor(getCorrectHueValue((float)(object.getDouble("color"))));
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	/**
	 * Obtains a correct value for the color based on the input paremeter and the correct range of HUE [0,360]
	 * @param c
	 * @return a valid HUE value
	 */
	public float getCorrectHueValue(float c) {
		if (c<0f) return 0f;
		if (c>360f) return 360f;
		return c;
	}

	/**
	 * @return the center
	 */
	public GPSPosition getCenter() {
		return center;
	}

	/**
	 * @param center the center to set
	 */
	public void setCenter(GPSPosition center) {
		this.center = center;
	}

	/**
	 * @return the actionRadiusMeters
	 */
	public double getActionRadiusMeters() {
		return actionRadiusMeters;
	}

	/**
	 * @param actionRadiusMeters the actionRadiusMeters to set
	 */
	public void setActionRadiusMeters(double actionRadiusMeters) {
		this.actionRadiusMeters = actionRadiusMeters;
	}

	/**
	 * @return the color
	 */
	public float getColor() {
		return color;
	}

	/**
	 * @param hue - the hue value to set
	 */
	public void setColor(float hue) {
		this.color = hue;
	}

}
