/**
 * 
 */
package uoc.tfc.gien.entities.interactive;

import java.io.File;
import java.io.Serializable;
//import java.net.URI;


import java.net.URL;

import org.json.JSONObject;

import uoc.tfc.gien.GlobalApplicationData;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;


/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class Resource implements Usable, Serializable {
	
	private static final long serialVersionUID = 1L;
	
	public enum ContentType {
		
		AUDIO, VIDEO, WEB, DOCUMENT, IMAGE;
		
	}
	
	private int id;
	private String name;
	private String description;
	private Uri uri;
	private String localFilePath;
	private ContentType contentType;
	
	/**
	 * Constructor
	 */
	public Resource() {
		//TO DO
	}
	
	/**
	 * Constructor
	 * 
	 * @param id
	 * @param name
	 * @param description
	 * @param uri
	 * @param localFilePath
	 * @param contentType
	 */
	public Resource(int id, String name, String description, Uri uri, String localFilePath, ContentType contentType) {
		this.setId(id);
		this.setName(name);
		this.setDescription(description);
		this.setUri(uri);
		this.setLocalFilePath(localFilePath);
		this.setContenttype(contentType);
	}
	
	/**
	 * Loads the object properties from a JSONObject
	 * 
	 * @param object
	 * @return true if success, false if not
	 */
	public boolean loadJSON(JSONObject object) {
		try {
			this.setId(object.getInt("id"));
			this.setName(object.getString("name"));
			this.setDescription(object.getString("description"));
			if (object.has("uri")) {
				this.setUri(Uri.parse((object.getString("uri"))));
			}
			this.setLocalFilePath(object.optString("localpath"));
			this.setContenttype(ContentType.valueOf(object.getString("contenttype")));
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
			return false;
		}
		return true;
	}

	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the uri
	 */
	public Uri getUri() {
		return uri;
	}

	/**
	 * @param uri the uri to set
	 */
	public void setUri(Uri uri) {
		this.uri = uri;
	}
	
	/**
	 * 
	 * @param strUri the string representation of the uri to set
	 */
	public void setUri(String strUri) {
		try {
			URL url = new URL(strUri);
			Uri uri = Uri.parse(url.toURI().toString()); // we need android.net.Uri, not java.net.URI
			this.uri = uri;
		} catch (Exception e){
			this.uri = null;
		}
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the contenttype
	 */
	public ContentType getContenttype() {
		return contentType;
	}

	/**
	 * @param contenttype the contenttype to set
	 */
	public void setContenttype(ContentType contentType) {
		this.contentType = contentType;
	}


	/**
	 * @return the localFilePath
	 */
	public String getLocalFilePath() {
		return localFilePath;
	}

	/**
	 * @param localFilePath the localFilePath to set
	 */
	public void setLocalFilePath(String localFilePath) {
		this.localFilePath = localFilePath;
	}

	@Override
	public void use(Context context) {
		// start an intent. The intent depends on the type of resource
		try {
			GlobalApplicationData app = (GlobalApplicationData) context.getApplicationContext();
			Intent intent;
			Uri correctUri;
			if (app.isOnlineGie()) {
				correctUri = this.uri;
			} else {
				if (getLocalFilePath()!=null) {
					correctUri = Uri.fromFile(new File(app.getGieFolder()+File.separator+"media"
							+File.separator+this.getContenttype().name()+File.separator+this.getLocalFilePath()));
				} else {
					return;
				}
			}
			switch (this.contentType) {
				case WEB:
					intent = new Intent(Intent.CATEGORY_BROWSABLE, correctUri);
					intent.setAction(Intent.ACTION_VIEW);
					//intent.addCategory(Intent.CATEGORY_DEFAULT);
					break;
				default:
					intent = new Intent(Intent.ACTION_VIEW, correctUri);
					break;
			}
			intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			context.getApplicationContext().startActivity(intent);
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
			
		}
	}
	
}
