/**
 * 
 */
package uoc.tfc.gien.entities.interactive;

import java.io.Serializable;
import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONObject;

//import android.util.Log;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class Select implements Serializable {
	
	//final String TAG = "Select";
	private static final long serialVersionUID = 1L;
	
	private int id;
	private String title;
	private String text;
	private ArrayList<Usable> items;
	
	/**
	 * Constructor
	 */
	public Select() {
		this.items = new ArrayList<Usable>();
	}
	
	/**
	 * Constructor
	 * 
	 * @param id
	 * @param title
	 * @param text
	 * @param items
	 */
	public Select(int id, String title, String text, ArrayList<Usable> items) {
		this.setId(id);
		this.setTitle(title);
		this.setText(text);
		this.setItems(items);
		if (this.getItems()==null) {
			this.setItems(new ArrayList<Usable>());
		}
	}
	
	/**
	 * Loads the object properties from a JSONObject
	 * 
	 * @param object
	 * @return true if success, false if not
	 */
	public boolean loadJSON(JSONObject object) {
		try {
			this.setId(object.getInt("id"));
			this.setTitle(object.getString("title"));
			this.setText(object.getString("text"));
			JSONArray arr = object.getJSONArray("item");
			if (arr.length()>0) {
				for (int i = 0; i<arr.length(); i++) {
					Usable u = null;
					JSONObject o = arr.getJSONObject(i);
					switch (Usable.Type.valueOf(o.getString("type"))) {
					case RESOURCE:
						u = new Resource();
						u.loadJSON(o);
						break;
					case INVENTORY_OBJECT:
						u = new InventoryObject();
						u.loadJSON(o);
						break;
					case OPTION:
						u = new Option();
						u.loadJSON(o);
						break;
					}
					if (u!=null) {
						this.items.add(u);
					}
				}
			}
			this.setItems(items);
			return true;
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
			return false;
		}
	}


	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}


	/**
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}


	/**
	 * @return the title
	 */
	public String getTitle() {
		return title;
	}


	/**
	 * @param title the title to set
	 */
	public void setTitle(String title) {
		this.title = title;
	}


	/**
	 * @return the text
	 */
	public String getText() {
		return text;
	}


	/**
	 * @param text the text to set
	 */
	public void setText(String text) {
		this.text = text;
	}


	/**
	 * @return the items
	 */
	public ArrayList<Usable> getItems() {
		return items;
	}


	/**
	 * @param items the items to set
	 */
	public void setItems(ArrayList<Usable> items) {
		this.items = items;
	}
	
}
