/**
 * 
 */
package uoc.tfc.gien.geofencing;

import java.util.Iterator;
import java.util.List;

import uoc.tfc.gien.GlobalApplicationData;
import uoc.tfc.gien.R;

import uoc.tfc.gien.communication.Communication;

import uoc.tfc.gien.entities.geo.SinglePointObject;

import uoc.tfc.gien.entities.interactive.Select;
import uoc.tfc.gien.interpreter.JSONResponseInterpreter;

import com.google.android.gms.location.Geofence;
import com.google.android.gms.location.GeofenceStatusCodes;
import com.google.android.gms.location.GeofencingEvent;

import android.app.IntentService;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.util.Log;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class GeofenceTransitionsIntentService extends IntentService {
	
	final String TAG = "GeofenceTransitionsIntentService";
	private Notification.Builder notificationBuilder;
	private NotificationManager notificationManager;
	
	private Uri notificationSoundUri;
	
	private GlobalApplicationData app;

	
	public GeofenceTransitionsIntentService() {
		super("GeofenceTransitionsIntentService");
	}
	
	@Override
	public void onCreate() {
		super.onCreate();
		
		app = (GlobalApplicationData) this.getApplicationContext();
		
		notificationBuilder = new Notification.Builder(app);
		notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
		notificationSoundUri = Uri.parse("android.resource://uoc.tfc.gien/" + R.raw.notification);
	}
	
	private String geofenceTransitionString(int code) {
		switch(code) {
			case Geofence.GEOFENCE_TRANSITION_ENTER: return "enter";
			case Geofence.GEOFENCE_TRANSITION_EXIT: return "exit";
			case Geofence.GEOFENCE_TRANSITION_DWELL: return "dwell";
			default: return "unknown";
		}
	}

	@Override
	protected void onHandleIntent(Intent intent) {
		try {
		GeofencingEvent geofencingEvent = GeofencingEvent.fromIntent(intent);
		if (geofencingEvent.hasError()) {
			Log.e(TAG,GeofenceStatusCodes.getStatusCodeString(geofencingEvent.getErrorCode()));
			return;
		}
		
		//Get the transition type.
		int geofenceTransition = geofencingEvent.getGeofenceTransition();

		//Test that the reported transition was of interest.
		if (geofenceTransition == Geofence.GEOFENCE_TRANSITION_DWELL || geofenceTransition == Geofence.GEOFENCE_TRANSITION_EXIT) {

			//Get the geofences that were triggered. A single event can trigger multiple geofences.
			List<Geofence> triggeringGeofences = geofencingEvent.getTriggeringGeofences();
			
			Iterator<Geofence> it = triggeringGeofences.iterator();
			Geofence g;
			
			while (it.hasNext()) {
				g = it.next();
				
				//if transition dwell create notification
				//else delete notification because we don't need notifications from other places different than here
				if (geofenceTransition == Geofence.GEOFENCE_TRANSITION_DWELL) {
				
					if (app.isOnlineGie()) {
						if (app.getExperience().isConnectOnGeoevent()) {
							String dataReceived = Communication.sendLocation(app.getLocation(), app.getServerUrl());
							Intent i = null;
							i = JSONResponseInterpreter.process(getBaseContext(), dataReceived);
							if (i!=null) {
								//getApplication().startActivity(i);
								doNotify(i,g,i.getStringExtra("type"));
							}
						}
					
					} else {
						if (Integer.valueOf(g.getRequestId())!=0) {
							int sId = app.getSPO(g.getRequestId()).getSelectId();
							if (sId!=0) {
								Select s = app.getDatabase().getSelect(sId);
								if (s!=null) {
									app.getQueuedSelects().put(s.getId(), s);
									intent = new Intent(getBaseContext(), uoc.tfc.gien.ui.SelectActivity.class);
									intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
									intent.putExtra("id", s.getId());
									doNotify(intent,g,"Selection");
								}
							}
						}
					}
				} else {
					if (geofenceTransition == Geofence.GEOFENCE_TRANSITION_EXIT) {
						notificationManager.cancel(Integer.parseInt(g.getRequestId()));
						app.getGeofenceSPOMap().remove(g.getRequestId());
					}
				}
				
			}
			
		} else {
			Log.e(TAG, "geofence transition invalid type: "+geofenceTransitionString(geofenceTransition));
		}
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}

	}
	
	private void doNotify(Intent i, Geofence g, String content) {
		SinglePointObject spo = app.getSPO(g.getRequestId());
		spo.setVisited(true);
		PendingIntent resultPendingIntent = PendingIntent.getActivity(getApplication(), 0, i, PendingIntent.FLAG_UPDATE_CURRENT);
		
		Notification.Builder nb = notificationBuilder
				.setSmallIcon(R.drawable.ic_notification)
				.setContentTitle(spo.getName())
				.setContentIntent(resultPendingIntent)
				.setAutoCancel(true);
		
		if (app.isSoundNotifications()) {
			nb.setSound(notificationSoundUri);
		}
		
		if (content!=null) {
			nb.setContentText(content);
		}
		
		Notification notification = nb.build();
		
		notificationManager.notify(Integer.parseInt(g.getRequestId()), notification);
	}
	

}
