/**
 * 
 */
package uoc.tfc.gien.geofencing;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;


import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
//import android.util.Log;

import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.Geofence;
import com.google.android.gms.location.GeofencingRequest;
import com.google.android.gms.location.LocationServices;

import uoc.tfc.gien.Constants;
import uoc.tfc.gien.GlobalApplicationData;
import uoc.tfc.gien.entities.geo.GPSPosition;
import uoc.tfc.gien.entities.geo.SinglePointObject;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class Geofencing {
	
	//final String TAG = "Geofencing";

	private Geofence.Builder geofenceBuilder;
	private List<Geofence> geofenceList;
	private PendingIntent geofencePendingIntent;
	private Context context;
	private GoogleApiClient googleApiClient;
	private GlobalApplicationData app;
	
	private static Geofencing instance;
	
	public static synchronized Geofencing getInstance(Context cntxt, GoogleApiClient gApiClient) {
		if (instance != null) {
			return instance;
		} else {
			instance = new Geofencing(cntxt, gApiClient);
			return instance;
		}
	}
	
	public static synchronized Geofencing getInstance() {
		if (instance != null) {
			return instance;
		} else {
			return null;
		}
	}
	
	/**
	 * Constructor
	 */
	private Geofencing(Context cntxt, GoogleApiClient gApiClient) {
		geofenceBuilder = new Geofence.Builder();
		geofenceList = new ArrayList<Geofence>();
		context = cntxt;
		googleApiClient = gApiClient;
		app = (GlobalApplicationData)context.getApplicationContext();
	}
	
	
	/**
	 * Creates a Geofence from a SinglePointObject
	 * 
	 * @param obj
	 */
	public Geofence createGeofence(SinglePointObject obj) {
		GPSPosition center = obj.getCenter();
		return geofenceBuilder.setRequestId(String.valueOf(obj.getId()))
		.setCircularRegion(center.getLatitude(), center.getLongitude(), (float)obj.getActionRadiusMeters())
		.setExpirationDuration(Geofence.NEVER_EXPIRE)
		.setLoiteringDelay(Constants.LOITERING_DELAY_MS) // time used for trigger a DWELL event
		//.setTransitionTypes(Geofence.GEOFENCE_TRANSITION_ENTER|Geofence.GEOFENCE_TRANSITION_EXIT)
		.setTransitionTypes(Geofence.GEOFENCE_TRANSITION_DWELL|Geofence.GEOFENCE_TRANSITION_EXIT)
		.build();
	}
	
	/**
	 * Adds a Geofence to the list
	 * 
	 * @param gf - the Geofence to add
	 */
	public void addGeofenceToList(Geofence gf) {
		geofenceList.add(gf);
	}
	
	public void addGeofence(SinglePointObject obj) {
		try {
			geofenceList.add(createGeofence(obj));
			app.getGeofenceSPOMap().put(String.valueOf(obj.getId()), obj);
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}
	}

	/**
	 * Creates a GeofencingRequest to specify the geofences to monitor and to set how related geofence events are triggered
	 * 
	 * @return a geofencing request
	 */
	public GeofencingRequest getGeofencingRequest() {
		GeofencingRequest.Builder builder = new GeofencingRequest.Builder();
		builder.setInitialTrigger(GeofencingRequest.INITIAL_TRIGGER_DWELL);
		builder.addGeofences(geofenceList);
		return builder.build();
	}
	
	/**
	 * 
	 * @return
	 */
	public PendingIntent getGeofencePendingIntent() {
		//Reuse the PendingIntent if we already have it
		if (geofencePendingIntent != null) {
			return geofencePendingIntent;
		}
		Intent intent = new Intent(context, GeofenceTransitionsIntentService.class);

		geofencePendingIntent = PendingIntent.getService(context, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT); 
        return geofencePendingIntent;
	}
	
	/**
	 * Removes geofences by their request IDs.
	 * @param geofences
	 */
	public void removeGeofences(ArrayList<String> geofences) {
		try {
			LocationServices.GeofencingApi.removeGeofences(googleApiClient, geofences);
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}
		Iterator<String> it = geofences.iterator();
		while (it.hasNext()) {
			app.getGeofenceSPOMap().remove(it.next());
		}
	}
	
	
	public void removeGeofences() {
		try {
			LocationServices.GeofencingApi.removeGeofences(googleApiClient, getGeofencePendingIntent());
		} catch (Exception e){
			//Log.e(TAG,e.getMessage());
		}
		app.getGeofenceSPOMap().clear();
	}

}
