/**
 * 
 */
package uoc.tfc.gien.interpreter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.regex.*;

import uoc.tfc.gien.GlobalApplicationData;
import uoc.tfc.gien.database.GieSQLite;
import uoc.tfc.gien.entities.interactive.Message;
import uoc.tfc.gien.entities.interactive.Select;
import android.content.Context;
import android.content.Intent;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class Interpreter {

	private enum Function {
		testMessage, showMessage, showSelect, testUpdateLocation, testPuntsCardinals, get, post
	}
	
	/**
	 * Constructor
	 */
	private Interpreter() {
		// TODO Auto-generated constructor stub
	}
	
	public static boolean isFunction(String strFunction) {
		Matcher m;
		String pattern = "^[A-Za-z0-9]+\\(.*\\)$";
		Pattern p = Pattern.compile(pattern);
		m = p.matcher(strFunction);
		return m.find();
	}
	
	public static String getFunctionName(String strFunction) {
		Matcher m;
		String pattern = "(^[A-Za-z0-9]+)\\(.*";
		Pattern p = Pattern.compile(pattern);
		m = p.matcher(strFunction);
		if (m.find()) {
			return m.group(1);
		} else {
			return null;
		}
	}
	
	public static String[] getFunctionParams(String strParams) {
		return strParams.split("\\,");
	}
	
	public static ArrayList<String> getFunctionComponents(String strFunction) {
		ArrayList<String> ret = new ArrayList<String>();
		String[] params = null;
		ret.add(getFunctionName(strFunction));
		if (ret.get(0)!=null) {
			params = getFunctionParams(strFunction.substring(ret.get(0).length()+1,strFunction.length()-1));
			for (int i=0;i<params.length;i++) {
				params[i] = params[i].trim();
			}
			ret.addAll(Arrays.asList(params));
			return ret;
		}
		else return null;
	}
	
	public static int execute(Context context, String function) {
		if (isFunction(function)) {
			ArrayList<String> arrFunction = getFunctionComponents(function);
			Intent intent = null;
			try {
				switch (Function.valueOf(arrFunction.get(0))) {
				case testMessage:
						Message msg = new Message();
						msg.setId(0);
						msg.setTitle("Test message");
						msg.setText("This message was created for testing the simpliest task of the interpreter module");
						msg.setTextType(Message.TextType.PLAIN_TEXT);
						intent = new Intent(context, uoc.tfc.gien.ui.TextMessageActivity.class);
						intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
						intent.putExtra("message", msg);
						context.startActivity(intent);
						break;
					case showMessage:
						if (arrFunction.get(1)!=null) {
							showMessage(context, Integer.valueOf(arrFunction.get(1)));
						}
						break;
					case showSelect:
						if (arrFunction.get(1)!=null) {
							showSelect(context, Integer.valueOf(arrFunction.get(1)));
						}
						break;
					case testUpdateLocation:
						intent = new Intent(context, uoc.tfc.gien.communication.SendLocationIntentService.class);
						context.startService(intent);
						break;
					case testPuntsCardinals:
						intent = new Intent(context, uoc.tfc.gien.TestPuntsCardinalsIntentService.class);
						context.startService(intent);
						break;
					case get:
						sendFunction(context, "GET", arrFunction);
						break;
					case post:
						sendFunction(context, "POST", arrFunction);
						break;
				}
				return 0;
			} catch (Exception e) {
				//Log.e(TAG,e.getMessage());
				return -1;
			}
		} else return -1;
	}
	
	private static void sendFunction(Context context, String method, ArrayList<String> arrFunction) {
		Intent intent = new Intent(context, uoc.tfc.gien.communication.SendFunctionIntentService.class);
		if ((method.compareTo("GET")==0)||(method.compareTo("POST")==0)) {
			intent.putExtra("method", method);
		} else {
			return;
		}
		intent.putStringArrayListExtra("functionComponents", arrFunction);
		context.startService(intent);
	}
	
	private static void showMessage(Context context, int id) {
		GlobalApplicationData app = (GlobalApplicationData) context.getApplicationContext();
		if (!app.isOnlineGie()) {
			GieSQLite db = GieSQLite.getInstance(context);
			Message m = db.getMessage(id);
			Intent intent = new Intent(context, uoc.tfc.gien.ui.TextMessageActivity.class);
			intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			intent.putExtra("message", m);
			context.startActivity(intent);
		}
	}
	
	private static void showSelect(Context context, int id) {
		GlobalApplicationData app = (GlobalApplicationData) context.getApplicationContext();
		if (!app.isOnlineGie()) {
			GieSQLite db = GieSQLite.getInstance(context);
			Select s = db.getSelect(id);
			app.getQueuedSelects().put(s.getId(), s);
			Intent intent = new Intent(context, uoc.tfc.gien.ui.SelectActivity.class);
			intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			intent.putExtra("id", id);
			context.startActivity(intent);
		}
	}

}
