/**
 * 
 */
package uoc.tfc.gien.interpreter;

import org.json.JSONException;
import org.json.JSONObject;

import uoc.tfc.gien.GlobalApplicationData;
import uoc.tfc.gien.entities.geo.Layer;
import uoc.tfc.gien.entities.interactive.Message;
import uoc.tfc.gien.entities.interactive.Resource;
import uoc.tfc.gien.entities.interactive.Select;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class JSONResponseInterpreter {
	
	final static String TAG = "Communication";
	
	private JSONResponseInterpreter() {

	}

	public static Intent process(Context context, String jsonData) {
		GlobalApplicationData app = (GlobalApplicationData) context.getApplicationContext();
		JSONObject rootObj;
		Intent intent = null;
		try {
			rootObj = new JSONObject(jsonData);
			if (rootObj.has("message")) {
				Message msg = new Message();
				if (msg.loadJSON(rootObj.getJSONObject("message"))) {
					switch (msg.getTextType()) {
						case PLAIN_TEXT:
							intent = new Intent(context, uoc.tfc.gien.ui.TextMessageActivity.class);
							intent.putExtra("message", msg);
							break;
						case HTML:
							intent = new Intent(context, uoc.tfc.gien.ui.HtmlMessageActivity.class);
							intent.putExtra("message", msg);
					}
					intent.putExtra("type", "Message");
					//return intent;
				}
			} else {
				if (rootObj.has("select")) {
					Select s = new Select();
					if (s.loadJSON(rootObj.getJSONObject("select"))) {
						app.getQueuedSelects().put(s.getId(), s);
						intent = new Intent(context, uoc.tfc.gien.ui.SelectActivity.class);
						intent.putExtra("id", s.getId());
						intent.putExtra("type", "Selection");
					}
				}
				else {
					if (rootObj.has("resource")) {
						Resource r = new Resource();
						if (r.loadJSON(rootObj.getJSONObject("resource"))) {
							switch (r.getContenttype()) {
							case WEB:
								intent = new Intent(Intent.CATEGORY_BROWSABLE, r.getUri());
								intent.setAction(Intent.ACTION_VIEW);
								//intent.addCategory(Intent.CATEGORY_DEFAULT);
								break;
							default:
								intent = new Intent(Intent.ACTION_VIEW, r.getUri());
								break;
							}
							intent.putExtra("type", r.getContenttype().name());
						}
					}
					else {
						if ((rootObj.has("layer"))||(rootObj.has("map"))) {
							
								intent = new Intent(context,uoc.tfc.gien.UpdateMapIntentService.class);
								intent.putExtra("data", jsonData);
						        context.getApplicationContext().startService(intent);
						    
						}
					}
				}
			}
		} catch (JSONException e) {
			Log.e(TAG,e.getMessage());
		}
		if (intent!=null) {
			intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		}
		return intent;
	}
	
}
