/**
 * 
 */
package uoc.tfc.gien.storage;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;

import uoc.tfc.gien.entities.GieListItem;
import android.content.res.Resources;

import android.os.Environment;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class Storage {


	private static File publicFolder;
	
	private Storage() {

	}
	
	public String getPublicFolder() {
		return publicFolder.toString();
	}
	
	
	/**
	 * Initializes the Storage object
	 * @return True if the app has access to the storage. False if not
	 */
	public static boolean init() {
		if (!Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)){
			//Log.d("gien", "No SDCARD");
			return false;
		} else {
			publicFolder = new File(Environment.getExternalStorageDirectory().toString(),File.separator+"Gies"); // Gies
			publicFolder.mkdirs();
			return true;
		}
	
	}
	
	/**
	 * Checks if the directory contains a gie
	 * @param directory - the directory to check
	 * @return True if the directory contains gie.json and info.json. False if not 
	 */
	private static boolean containsGie(File directory) {
		boolean containsGieJson = false;
		boolean containsInfoJson = false;
		File[] files = directory.listFiles();
		int i = 0;
		while ((i<files.length)&&(!(containsGieJson&&containsInfoJson))) {
			if (!containsGieJson) {
				if (files[i].getName().compareTo("gie.json")==0) {
					containsGieJson = true;
				}
			}
			if (!containsInfoJson) {
				if (files[i].getName().compareTo("info.json")==0) {
					containsInfoJson = true;
				}
			}
			i++;
		}
		if ((containsGieJson)&&(containsInfoJson)) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Obtains a list of directories with gies in the storage
	 * @return list of directories with gies 
	 */
	public static List<File> getPublicGies() {
		List<File> validFolders = new ArrayList<File>();		
		File[] files = publicFolder.listFiles();
		for (File f : files) {
			if (f.isDirectory()) {
				if (containsGie(f)) {
					validFolders.add(f);
				}
			}
		}	
		return validFolders;
	}
	
	/**
	 * 
	 * @param res
	 * @return
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static Vector<GieListItem> getPublicGies(Resources res) throws FileNotFoundException, IOException {
		Vector<GieListItem> v = new Vector<GieListItem>();
		File[] files = publicFolder.listFiles();
		for (File f : files) {
			if (f.isDirectory()) {
				if (containsGie(f)) {
					v.add(new GieListItem(f,res));
				}
			}
		}
		return v;
	}
	
	/**
	 * Loads and returns the content of a text file without new-line character
	 * @param f - the file
	 * @return A string with the content 
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private static String getContent(File f) {
		if (f.isFile()) {
			FileReader fr = null;
			BufferedReader br = null;
			StringBuffer content = new StringBuffer();
			try {
				String linea;
				fr = new FileReader(f);
				br = new BufferedReader(fr);
				while ((linea=br.readLine())!=null) {
					content.append(linea);
				}
				//br.close();
				//return content.toString();
			} catch (FileNotFoundException e) {
				//Log.d("gien-Storage-getContent",e.getMessage());
			} catch (IOException e1) {
				//Log.d("gien-Storage-getContent",e1.getMessage());
			} finally {
				try {           
					if( null != fr ){   
						fr.close();     
					}       
				} catch (Exception e2){ 
					//Log.d("gien-Storage-getContent",e2.getMessage());
				}
			}
			return content.toString();
		} else {
			return "";
		}
	}
	
	/**
	 * Loads and returns the content of info.json file without new-line character
	 * @param directory - the directory where the file is stored
	 * @return A string with the content
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static String getInfoJsonContent(File directory) throws FileNotFoundException, IOException {
		return getContent(new File(directory,File.separator+"info.json"));
	}
	
	/**
	 * Loads and returns the content of gie.json file without new-line character
	 * @param directory
	 * @return
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static String getGieJsonContent(File directory) throws FileNotFoundException, IOException {
		return getContent(new File(directory, File.separator+"gie.json"));
	}
	
	
}
