package uoc.tfc.gien.ui;

import java.net.URL;

import uoc.tfc.gien.GienException;
import uoc.tfc.gien.GlobalApplicationData;
import uoc.tfc.gien.LoadOnlineGie;
import uoc.tfc.gien.R;
import uoc.tfc.gien.entities.Favorite;
import uoc.tfc.gien.ui.listAdapters.FavouriteAdapter;
import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
//import android.util.Log;
import android.widget.Toast;

/**
 * 
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class FavoritesListActivity extends ListActivity {
	
	//final String TAG = "FavoritesListActivity";
	private Favorite selectedFavorite;
	private GlobalApplicationData app;
	private FavouriteAdapter adapter;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		app = (GlobalApplicationData)getApplicationContext();
		setContentView(R.layout.activity_favourites_list);
		adapter = new FavouriteAdapter(this,app.getFavoritesDatabase().getListReg());
		setListAdapter(adapter);
		registerForContextMenu(this.getListView());
	}
	
	@Override
	protected void onListItemClick(ListView listView, View view, int position, long id) {
		super.onListItemClick(listView, view, position, id);
		Object o = getListAdapter().getItem(position);
		try {
			new LoadOnlineGie(this).execute(new URL(((Favorite)o).getUrl().toString()));
			app.setFromFavorites(true);
		} catch (Exception e) {
			//Log.e(TAG,e.getMessage());
		}
	}
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {  
    	ListView listV = (ListView)v; // get the list view
    	AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo)menuInfo;
    	int position = info.position;  // get the list item position
    	this.selectedFavorite = (Favorite)listV.getItemAtPosition(position); // get the list item
    	
    	// configure the menu
    	menu.setHeaderTitle(selectedFavorite.getName());
    	menu.add(Menu.NONE, 0, 0, R.string.action_edit);
    	menu.add(Menu.NONE, 1, 1, R.string.action_delete);
	}
	
	@Override
	public boolean onContextItemSelected(MenuItem item) {
  		int menuItemIndex = item.getItemId();
  		switch (menuItemIndex) {
  			case 0: // edit
  				openFavoriteActivity();
  				break;
  			case 1: // delete
  				showConfirmationDialog();
  				break;
  		}
  		return true;
	}
	
	public void openFavoriteActivity() {
		Intent i = new Intent(this, uoc.tfc.gien.ui.FavoriteActivity.class);
		i.putExtra("id", selectedFavorite.getId());
		i.putExtra("name", selectedFavorite.getName());
		i.putExtra("url", selectedFavorite.getUrl().toString());
		i.putExtra("username", selectedFavorite.getUsername());
		i.putExtra("password",  selectedFavorite.getPassword());
		i.putExtra("edit", true);
		startActivity(i);
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		updateListView();
	}
	
	public void updateListView() {
		adapter.refresh(app.getFavoritesDatabase().getListReg());
	}
	
	public void deleteFavorite() {
		try {
			app.getFavoritesDatabase().deleteReg(selectedFavorite.getId());
			updateListView();
		} catch (GienException e) {
			Toast t = Toast.makeText(this, e.getMessage(), Toast.LENGTH_LONG);
			t.setGravity(Gravity.CENTER_HORIZONTAL|Gravity.CENTER_VERTICAL, 0, 0);
			t.show();
		}
	}
	
	public void showConfirmationDialog() {
		AlertDialog.Builder builder = new AlertDialog.Builder(FavoritesListActivity.this);
		builder.setMessage(R.string.confirmation_delete_favorite);
		
		builder.setPositiveButton(R.string.action_delete, new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int id) {
				deleteFavorite();
				//updateListView();
			}
		});
		
		builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				finish();
			}
		});
		
		AlertDialog dialog = builder.create();
		dialog.show();
	}


}
