package uoc.tfc.gien.ui;

import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

import org.json.JSONObject;

import uoc.tfc.gien.GlobalApplicationData;
import uoc.tfc.gien.R;
import uoc.tfc.gien.communication.Communication;
import uoc.tfc.gien.communication.RESTMethod;
import uoc.tfc.gien.entities.interactive.InventoryObject;
import uoc.tfc.gien.ui.listAdapters.InventoryObjectAdapter;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.res.Resources;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Html;
import android.view.View;
import android.widget.ListView;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;


/**
 * 
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class InventoryActivity extends ListActivity {
	
	private GlobalApplicationData globalData;
	private List<InventoryObject> inventory;
	private TreeMap<String,String> score;
	private InventoryObjectAdapter adapter;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_inventory);
		globalData = (GlobalApplicationData) getApplicationContext();
		score = globalData.getExperience().getScoreTable();
		inventory = globalData.getExperience().getInventory();
		reloadScoreTable();
		this.adapter = new InventoryObjectAdapter(this,inventory);
		setListAdapter(adapter);
		dataUpdate();
	}
	
	private void reloadScoreTable() {
		TableLayout scoreTable = (TableLayout)findViewById(R.id.table_score);
		scoreTable.removeAllViews();
		Iterator<String> it = score.keySet().iterator();
		while (it.hasNext()) {
			String key = it.next();
			TextView text = new TextView(this);
			text.setText(Html.fromHtml("<b>"+key+"</b>: "+score.get(key)));
			TableRow row = new TableRow(this);
			row.addView(text);
			scoreTable.addView(row);
		}
	}
	
	private void dataUpdate() {
		new UpdateTask(this).execute();
	}
	
	private void updateUI() {
		reloadScoreTable(); //update tableview with updated data
		adapter.notifyDataSetChanged(); //update list
	}
	
	private class UpdateTask extends AsyncTask<Void, Void, Boolean> {

		private ProgressDialog pd;
		private Context context;
		private Resources resources;
		final GlobalApplicationData app;
		
		
		public UpdateTask(Context context) {
			this.context = context;
			this.context = context;
			resources = context.getResources();
			app = (GlobalApplicationData) context.getApplicationContext();
		}
		
		@Override
		protected void onPreExecute() {
			pd = new ProgressDialog(context);
			pd.setProgressStyle(ProgressDialog.STYLE_SPINNER);
			pd.setCancelable(false);
			pd.setIndeterminate(true);
			pd.show();
	        super.onPreExecute();
		}
		
		@Override
		protected Boolean doInBackground(Void... params) {
			String data = null;
			boolean isInvLoaded = false;
			boolean isScoLoaded = false;
			if (app.isOnlineGie()) {
				try {
					data = Communication.getInvAndScore(app.getLocation(),app.getServerUrl());
					JSONObject o = new JSONObject(data);
					isInvLoaded = app.getExperience().loadJSONInventory(o);
					isScoLoaded = app.getExperience().loadJSONScoretable(o);
				} catch (Exception e) {
					return false;
				}
				return (isInvLoaded||isScoLoaded);
			} else {
				// Not applicable to offline gie. Do nothing
				return true;
			}
		}
		
		@Override
		protected void onPostExecute(Boolean success) {
			pd.dismiss();
			if (success) {
				updateUI();
			}
		}
	}
	
	@Override
	protected void onListItemClick(ListView listView, View view, int position, long id) {
		super.onListItemClick(listView, view, position, id);
		this.inventory.get(position).use(this);
	}
}
