package uoc.tfc.gien.ui;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;

import uoc.tfc.gien.CoreService;
import uoc.tfc.gien.GlobalApplicationData;
import uoc.tfc.gien.R;
import uoc.tfc.gien.database.GieSQLite;
import uoc.tfc.gien.entities.Experience;
import uoc.tfc.gien.entities.geo.GPSPosition;
import uoc.tfc.gien.entities.geo.GeoObject;
import uoc.tfc.gien.entities.geo.Layer;
import uoc.tfc.gien.entities.geo.Map;
import uoc.tfc.gien.entities.geo.MultiPointObject;
import uoc.tfc.gien.entities.geo.Polygon;
import uoc.tfc.gien.entities.geo.Route;
import uoc.tfc.gien.entities.geo.SinglePointObject;
import uoc.tfc.gien.entities.interactive.Select;
import uoc.tfc.gien.Constants;
import uoc.tfc.gien.GienException;
import uoc.tfc.gien.TouchedLocationAsyncTask;

import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMapLoadedCallback;
import com.google.android.gms.maps.GoogleMap.OnMarkerClickListener;
import com.google.android.gms.maps.MapFragment;
import com.google.android.gms.maps.UiSettings;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.CircleOptions;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.PolygonOptions;
import com.google.android.gms.maps.model.PolylineOptions;

import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Toast;

/**
 * 
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */

public class MapActivity extends FragmentActivity implements OnMapLoadedCallback, OnMarkerClickListener, SensorEventListener {

	private BroadcastReceiver receiver;
	private BroadcastReceiver receiverMapChanging;
	private GoogleMap gMap;
	private CameraUpdate cameraUpdate;
	private TreeMap<String, SinglePointObject> markers;
	private GlobalApplicationData app;
	private SensorManager sensorManager;
	
	private float[] gravity;
	private float[] geomagnetic;
	private float azimuth;
	private boolean firstTimeCalculatingBearing = true;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_map);
		app = (GlobalApplicationData) getApplication();
		markers = new TreeMap<String, SinglePointObject>();
		Experience exp = app.getExperience();
		
		//map
		MapFragment mapFragment = (MapFragment) getFragmentManager().findFragmentById(R.id.map);
		gMap = mapFragment.getMap();
		gMap.setMyLocationEnabled(true);
		UiSettings uiSettings = gMap.getUiSettings();
		uiSettings.setCompassEnabled(exp.isCompassEnabled());
		uiSettings.setMapToolbarEnabled(exp.isMapToolbarEnabled());
		uiSettings.setAllGesturesEnabled(false);
		uiSettings.setZoomControlsEnabled(false);
		uiSettings.setMyLocationButtonEnabled(false);
		cameraUpdate = CameraUpdateFactory.zoomTo(getValidZoomLevel(exp.getZoomLevel()));
		gMap.moveCamera(cameraUpdate);
		
		sensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
		
		//register a receiver for change the map center in case CoreService detects a location change
		IntentFilter intentFilter = new IntentFilter();
		intentFilter.addAction("uoc.tfc.gien.LOCATION_CHANGE");
		receiver = new BroadcastReceiver() {
			@Override
			public void onReceive(Context context, Intent intent) {
				double[] location = intent.getDoubleArrayExtra("location");
				cameraUpdate = CameraUpdateFactory.newLatLng(new LatLng(location[0],location[1]));
				gMap.moveCamera(cameraUpdate);
			}
			
		};
		registerReceiver(receiver, intentFilter);
		
		
		IntentFilter intentFilterMapChanging = new IntentFilter();
		intentFilterMapChanging.addAction("uoc.tfc.gien.MAP_CHANGING");
		receiverMapChanging = new BroadcastReceiver() {
			@Override
			public void onReceive(Context context, Intent intent) {
				if (intent.getStringExtra("status").compareTo("start")==0) {
					deactivateMarkerClick();
				} else {
					if (intent.getStringExtra("status").compareTo("end")==0) {
						gMap.clear();
						drawLayers();
						activateMarkerClick();
					}
				}
			}
		};
		registerReceiver(receiverMapChanging, intentFilterMapChanging);

		activateMarkerClick();
		
		drawLayers();
		
	}
	
	public void activateMarkerClick() {
		gMap.setOnMarkerClickListener(this);
	}
	
	public void deactivateMarkerClick() {
		gMap.setOnMarkerClickListener(null);
	}
	
	/**
	 * Adjusts the zoom level value for use it in the GoogleMap. Valid values are in range [2.0,21.0]
	 *  
	 * @param zoom
	 * @return a valid zoom level based on the input parameter
	 */
	public float getValidZoomLevel(float zoom) {
		if (zoom<2.0f) return 2.0f;
		if (zoom>21.0f) return 21.0f;
		return zoom;
	}
	
	/**
	 * Draws all the visible layers of the map
	 */
	public void drawLayers() {
		Map map = app.getExperience().getMap();
		Layer layer;
		if (map!=null) {
			ArrayList<Layer> layers = map.getLayers();
			Iterator<Layer> it = layers.iterator();
			while (it.hasNext()) {
				layer = it.next();
				if (layer.isVisible()) {
					drawLayer(layer);
				}
			}
		}
	}
	
	/**
	 * Draws the specified layer 
	 * 
	 * @param layer - the layer object to draw
	 */
	public void drawLayer(Layer layer) {	
		if (layer!=null) {
			ArrayList<GeoObject> objects = (ArrayList<GeoObject>)layer.getObjects();
			Iterator<GeoObject> it = objects.iterator();
			while (it.hasNext()) {
				GeoObject go = it.next();
				if (go instanceof SinglePointObject) {
					drawSPObject((SinglePointObject)go);
				} else {
					if (go instanceof Polygon) {
						drawPolygon((Polygon)go, layer.getZindex(), false);
					} else {
						if (go instanceof Route) {
							drawRoute((Route)go, layer.getZindex(), true);
						} else {
							if (go instanceof MultiPointObject) {
								drawMPObject((MultiPointObject)go);
							}
						}
					}
				}
			}
		}
	}
	
	/**
	 * Draws a SinglePointObject in the map
	 * 
	 * @param obj
	 */
	public void drawSPObject(SinglePointObject obj) {
		if (obj.isVisible()) {
			MarkerOptions mo = new MarkerOptions().alpha(1f)
												.icon(BitmapDescriptorFactory.defaultMarker(obj.getColor()))
												.position(new LatLng(obj.getCenter().getLatitude(), obj.getCenter().getLongitude()));
			markers.put(gMap.addMarker(mo).getId(),obj);
		}
	}
	
	/**
	 * Draws a MultiPointObject in the map
	 * 
	 * @param obj
	 */
	public void drawMPObject(MultiPointObject obj) {
		if (obj.isVisible()) {
			ArrayList<SinglePointObject> points = (ArrayList<SinglePointObject>)obj.getPoints();
			if (!points.isEmpty()) {
				Iterator<SinglePointObject> it = points.iterator();
				while (it.hasNext()) {
					SinglePointObject spo = it.next();
					drawSPObject(spo);
				}
			}
		}
	}
	
	/**
	 * Draws a route in the map
	 * 
	 * @param r
	 * @param zindex
	 * @param drawPoints
	 */
	public void drawRoute(Route r, int zindex, boolean drawPoints) {
		if (r.isVisible()) {
			ArrayList<SinglePointObject> points = (ArrayList<SinglePointObject>)r.getPoints();
			if (!points.isEmpty()) {
				Iterator<SinglePointObject> it = points.iterator();
				ArrayList<LatLng> gPoints = new ArrayList<LatLng>();
				while (it.hasNext()) {
					SinglePointObject spo = it.next();
					GPSPosition center = spo.getCenter();
					LatLng coord = new LatLng(center.getLatitude(),center.getLongitude());
					gPoints.add(coord);
					if (drawPoints) {
						CircleOptions circleOptions = new CircleOptions();
						circleOptions.center(coord)
									.fillColor(r.getColor())
									.radius(2)
									.strokeWidth(0)
									.zIndex(zindex);
						gMap.addCircle(circleOptions);
					}
				}
				PolylineOptions polyOptions = new PolylineOptions();
				polyOptions.addAll(gPoints)
							.color(r.getColor()) //32bit ARGB .color(0xffff0000)
							.visible(true)
							.zIndex(zindex)
							.width((float)r.getWidth());
				gMap.addPolyline(polyOptions);
			}
		}
	}
	
	/**
	 * Draws a polygon in the map
	 * 
	 * @param p
	 * @param zindex
	 * @param drawPoints
	 */
	public void drawPolygon(Polygon p, int zindex, boolean drawPoints) {
		if (p.isVisible()) {
			ArrayList<SinglePointObject> points = (ArrayList<SinglePointObject>)p.getPoints();
			if (!points.isEmpty()) {
				Iterator<SinglePointObject> it = points.iterator();
				ArrayList<LatLng> gPoints = new ArrayList<LatLng>();
				while (it.hasNext()) {
					SinglePointObject spo = it.next();
					GPSPosition center = spo.getCenter();
					LatLng coord = new LatLng(center.getLatitude(),center.getLongitude());
					gPoints.add(coord);
					if (drawPoints) {
						CircleOptions circleOptions = new CircleOptions();
						circleOptions.center(coord)
									.fillColor(p.getColor())
									.radius(2)
									.zIndex(zindex)
									.strokeWidth(0);
						gMap.addCircle(circleOptions);
					}
				}
				PolygonOptions polygonOptions = new PolygonOptions();
				polygonOptions.addAll(gPoints)
							.fillColor(p.getColor()) //32bit ARGB .color(0xffff0000)
							.visible(true)
							.zIndex(zindex)
							.strokeWidth(0);
				gMap.addPolygon(polygonOptions);
			}
		}
	}
	

	@Override
	public void onDestroy() {
		unregisterReceiver(receiver);
		unregisterReceiver(receiverMapChanging);
		sensorManager.unregisterListener(this);
		super.onDestroy();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.map_actionbar, menu);
		return super.onCreateOptionsMenu(menu);
	}
	
	public boolean onPrepareOptionsMenu(Menu menu) {
		// hide/show menu items properly
		MenuItem menuItem = menu.findItem(R.id.action_actions);
		menuItem.setVisible(app.getExperience().getActionsVisible());
		menuItem = menu.findItem(R.id.action_inventory);
		if (app.isOnlineGie()) {
			menuItem.setVisible(app.getExperience().getInventoryVisible());
		} else {
			menuItem.setVisible(false); // offline experience never have inventory
		}
		menuItem = menu.findItem(R.id.action_add_to_favorites);
		if ( (!app.isOnlineGie()) || (app.isFromFavorites()) ) {
			menuItem.setEnabled(false);
		}
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case R.id.action_inventory:
				openInventory();
				return true;
			case R.id.action_actions:
				openActions();
				return true;
			case R.id.action_settings:
				openSettings();
				return true;
			case R.id.action_gie_details:
				openGieInfo();
				return true;
			case R.id.action_add_to_favorites:
				try {
					app.getFavoritesDatabase().addReg(app.getServerUrl(), null, null, app.getGieInfo().getName());
					inform(getString(R.string.favorite_saved_successfully));
				} catch (GienException e) {
					inform(e.getMessage());
				} finally {
					item.setEnabled(false);
				}
				return true;
			case R.id.action_stop_gie:
				openStopDialog();
				return true;
			case R.id.action_about:
				openAboutDialog();
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}

	}
	
	public void openInventory() {
		startActivity(new Intent(MapActivity.this,InventoryActivity.class));
	}
	
	public void openActions() {
		startActivity(new Intent(MapActivity.this,ActionsListActivity.class));
	}
	
	public void openGieInfo() {
		startActivity(new Intent(MapActivity.this,GieInfoActivity.class));
	}
	
	public void openAboutDialog() {
		View view = getLayoutInflater().inflate(R.layout.activity_about, null, false);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setIcon(R.drawable.ic_launcher)
        	.setTitle(R.string.app_name)
        	.setView(view)
        	.create()
        	.show();
	}
	
	public void openSettings() {
		Intent intent = new Intent(this, PreferencesActivity.class);
		try {
			startActivity(intent);
		} catch (Exception e) {
			showDialog(e.getMessage());
		}
	}
	
	public void stopGie() {
		stopService(new Intent(this, CoreService.class));
		GieSQLite db = app.getDatabase();
		db.clean();
		finish();
		Intent i = new Intent(getApplicationContext(), MainActivity.class);
		startActivity(i);
		//finish();
	}
	
	public void openStopDialog() {
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setIconAttribute(android.R.attr.alertDialogIcon)
			.setTitle(R.string.dialog_stop_title)
			.setMessage(R.string.dialog_stop_message);
		builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int id) {
				stopGie();
			}
		});
		builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int id) {
				dialog.dismiss();
			}
		});
		builder.create()
			.show();
	}
	
	public void inform(String msg) {
		Toast t = Toast.makeText(this, msg, Toast.LENGTH_LONG);
		t.setGravity(Gravity.CENTER_HORIZONTAL|Gravity.CENTER_VERTICAL, 0, 0);
		t.show();
	}
	
	//Debug
	public void showDialog(String msg) {
		Toast.makeText(this, "Gien - " + msg, Toast.LENGTH_SHORT).show();
	}
	
	//Debug
	public void changeLocation(double latitude, double longitude) {
		showDialog("latitude:" + Double.toString(latitude) + " - longitude:" + Double.toString(longitude));
	}


	@Override
	public void onMapLoaded() {
		// TODO Auto-generated method stub
		//inform("Map loaded"); //debug
	}

	@Override
	public boolean onMarkerClick(Marker marker) {
		//inform(markers.get(marker.getId()).getName()); //debug
		
		int selectId = markers.get(marker.getId()).getSelectId();
		
		if (app.isOnlineGie()) {
			// enviar consulta
			new TouchedLocationAsyncTask(MapActivity.this).execute(markers.get(marker.getId()).getId());
			
		} else {
			if (selectId!=0) {
				// obrir el select
				Select s = app.getDatabase().getSelect(selectId);
				if (s!=null) {
					app.getQueuedSelects().put(s.getId(), s);
					Intent intent = new Intent(getBaseContext(), uoc.tfc.gien.ui.SelectActivity.class);
					intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
					intent.putExtra("id", s.getId());
					startActivity(intent);
				}
			}
		}
		
		return true;
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		
		//for the system's orientation sensor registered listeners
		sensorManager.registerListener(this, sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD), SensorManager.SENSOR_DELAY_UI); 
		sensorManager.registerListener(this, sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER), SensorManager.SENSOR_DELAY_UI); // microsegundos 10^-6
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		sensorManager.unregisterListener(this);
	}

	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		// TODO Auto-generated method stub
		
	}

	
	@Override
	public void onSensorChanged(SensorEvent event) {
		// Calculates the azimuth using accelerometer and magnetic field sensors
		if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER)
            gravity = event.values;
        if (event.sensor.getType() == Sensor.TYPE_MAGNETIC_FIELD)
            geomagnetic = event.values;
        if (gravity != null && geomagnetic != null) {
            float R[] = new float[9];
            float I[] = new float[9];
            boolean success = SensorManager.getRotationMatrix(R, I, gravity, geomagnetic);
            if (success) {
                float orientationData[] = new float[3];
                SensorManager.getOrientation(R, orientationData);
                
                // saves the starting value for first comparing
                if (firstTimeCalculatingBearing) {
                	azimuth = (float)Math.toDegrees(orientationData[0]);
                	firstTimeCalculatingBearing = false;
                }
                
                float newAzimuth = (float)Math.toDegrees(orientationData[0]); // converts azimuth angle from radians to degrees
                
                // applies a rotation to the map if the azimuth has changed significantly
                if (Math.abs(newAzimuth-azimuth) > Constants.AZIMUTH_SIGNIFICANT_CHANGE_DG) {
                	GPSPosition pos = app.getLocation();
                    CameraPosition cPos = CameraPosition.builder()
                    		.target(new LatLng(pos.getLatitude(), pos.getLongitude()))
                    		.bearing(newAzimuth)
                    		.zoom(app.getExperience().getZoomLevel())
                    		.build();

                    // smooths the rotation with an animation
                    gMap.animateCamera(CameraUpdateFactory.newCameraPosition(cPos), Constants.MAP_ROTATION_ANIMATION_TIME_MS, null);
                    
                    azimuth = newAzimuth;
                }  
            }
        }
		
	}
	
}
