/**
 * 
 */
package uoc.tfc.gien.ui.listAdapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.List;

import uoc.tfc.gien.R;
import uoc.tfc.gien.entities.interactive.InventoryObject;
import uoc.tfc.gien.entities.interactive.Option;
import uoc.tfc.gien.entities.interactive.Resource;
import uoc.tfc.gien.entities.interactive.Usable;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class SelectAdapter extends BaseAdapter {

	private final Activity activity;
	private final List<Usable> list;
	
	/**
	 * Constructor
	 * 
	 * @param activity
	 * @param list
	 */
	public SelectAdapter(Activity activity, List<Usable> list) {
		super();
		this.activity = activity;
		this.list = list;
	}
	
	@Override
	public int getCount() {
		return list.size();
	}

	@Override
	public Object getItem(int position) {
		return list.get(position);
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		LayoutInflater inflater = activity.getLayoutInflater();
		View view = inflater.inflate(R.layout.select_row_layout, null, true);
		TextView name = (TextView)view.findViewById(R.id.name);
		TextView description = (TextView)view.findViewById(R.id.description);
		ImageView image = (ImageView)view.findViewById(R.id.icon);
		Usable u = list.get(position);
		if (u instanceof Resource) {
			Resource r = (Resource)list.get(position);
			name.setText(r.getName());
			description.setText(r.getDescription());
			switch (r.getContenttype()) {
				case AUDIO:
					image.setImageResource(R.drawable.ic_audio);
					break;
				case VIDEO:
					image.setImageResource(R.drawable.ic_movie);
					break;
				case WEB:
					image.setImageResource(R.drawable.ic_web);
					break;
				case IMAGE:
					image.setImageResource(R.drawable.ic_picture);
					break;
				case DOCUMENT:
					image.setImageResource(R.drawable.ic_document);
					break;
			}
		} else {
			if (u instanceof InventoryObject) {
				InventoryObject io = (InventoryObject)list.get(position); 
				name.setText(io.getName());
				description.setText(io.getDescription());
				image.setImageResource(R.drawable.ic_box);
			}
			else {
				if (u instanceof Option) {
					Option o = (Option)list.get(position);
					name.setText(o.getName());
					description.setText(o.getDescription());
					image.setImageResource(R.drawable.ic_hand);
				}
			}
		}
		return view;		 
	}

}
